"""
 Copyright (C) 2020 Xilinx, Inc

 Ctypes based based bandwidth testcase used with every platform as part of
 xbutil validate

 Licensed under the Apache License, Version 2.0 (the "License"). You may
 not use this file except in compliance with the License. A copy of the
 License is located at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 License for the specific language governing permissions and limitations
 under the License.
"""

import ctypes.util
import sys
import time
import math

# Following found in PYTHONPATH setup by XRT
from xrt_binding import *
from ert_binding import *

# utils_binding.py
sys.path.append('../')
from utils_binding import *

# Define libc helpers
libc_name = ctypes.util.find_library("c")
libc = ctypes.CDLL(libc_name)
libc.memcmp.argtypes = (ctypes.c_void_p, ctypes.c_void_p, ctypes.c_size_t)
libc.memcmp.restype = (ctypes.c_int)
libc.memcpy.argtypes = (ctypes.c_void_p, ctypes.c_void_p, ctypes.c_size_t)
libc.memcpy.restype = (ctypes.c_void_p)

current_micro_time = lambda: int(round(time.time() * 1000000))

globalbuffersize = 1024*1024*16    #16 MB

def getThreshold(devHandle):
    threshold = 40000
    deviceInfo = xclDeviceInfo2()
    xclGetDeviceInfo2(devHandle, ctypes.byref(deviceInfo))
    if b"qdma" in deviceInfo.mName or b"qep" in deviceInfo.mName:
        threshold = 30000
    if b"gen3x4" in deviceInfo.mName or b"_u26z_" in deviceInfo.mName:
        threshold = 20000
    if b"u2x4" in deviceInfo.mName or b"U2x4" in deviceInfo.mName or b"u2_gen3x4" in deviceInfo.mName:
        threshold = 10000
    if b"_u25_" in deviceInfo.mName or b"_u30_" in deviceInfo.mName: # so that it doesn't set theshold for u250
        threshold = 9000
    return threshold

def getInputOutputBuffer(devhdl, krnlhdl, argno, isInput):
    grpid = xrtKernelArgGroupId(krnlhdl, argno)
    if grpid < 0:
        raise RuntimeError("failed to find BO group ID: %d" % grpid)
    bo = xrtBOAlloc(devhdl, globalbuffersize, 0, grpid)
    if bo == 0:
        raise RuntimeError("failed to alloc buffer")

    bobuf = xrtBOMap(bo)
    if bobuf == 0:
        raise RuntimeError("failed to map buffer")
    lst = ctypes.cast(bobuf, ctypes.POINTER(ctypes.c_char))

    for i in range(globalbuffersize):
        lst[i] = i%256 if isInput else 0

    xrtBOSync(bo, xclBOSyncDirection.XCL_BO_SYNC_BO_TO_DEVICE, globalbuffersize, 0)
    return bo, bobuf

def runKernel(opt):
    khandle1 = xrtPLKernelOpen(opt.handle, opt.xuuid, "bandwidth1")
    khandle2 = xrtPLKernelOpen(opt.handle, opt.xuuid, "bandwidth2")
    kfunc = xrtKernelGetFunc(xrtBufferHandle, xrtBufferHandle, ctypes.c_int, ctypes.c_int)

    output_bo1, output_buf1 = getInputOutputBuffer(opt.handle, khandle1, 0, False)
    output_bo2, output_buf2 = getInputOutputBuffer(opt.handle, khandle2, 0, False)
    input_bo1, input_buf1 = getInputOutputBuffer(opt.handle, khandle1, 1, True)
    input_bo2, input_buf2 = getInputOutputBuffer(opt.handle, khandle2, 1, True)

    typesize = 512
    threshold = getThreshold(opt.xcl_handle)
    globalbuffersizeinbeats = globalbuffersize/(typesize>>3)
    tests= int(math.log(globalbuffersizeinbeats, 2.0))+1
    beats = 16

    #lists
    dnsduration = []
    dsduration  = []
    dbytes      = []
    dmbytes     = []
    bpersec     = []
    mbpersec    = []

    #run tests with burst length 1 beat to globalbuffersize
    #double burst length each test
    test=0
    throughput = []
    failed = False
    while beats <= 1024 and not failed:
        print("LOOP PIPELINE %d beats" %beats)

        usduration = 0
        fiveseconds = 5*1000000
        reps = 64
        while usduration < fiveseconds:
            start = current_micro_time()
            rhandle1 = kfunc(khandle1, output_bo1, input_bo1, beats, reps)
            rhandle2 = kfunc(khandle2, output_bo2, input_bo2, beats, reps)
            xrtRunWait(rhandle1)
            xrtRunWait(rhandle2)
            end = current_micro_time()

            xrtRunClose(rhandle1)
            xrtRunClose(rhandle2)

            usduration = end-start

            limit = beats*(typesize>>3)
            xrtBOSync(output_bo1, xclBOSyncDirection.XCL_BO_SYNC_BO_FROM_DEVICE, limit, 0)
            xrtBOSync(output_bo2, xclBOSyncDirection.XCL_BO_SYNC_BO_FROM_DEVICE, limit, 0)
            if libc.memcmp(input_buf1, output_buf1, limit) or libc.memcmp(input_buf2, output_buf2, limit):
               failed = True
               break

            # print("Reps = %d, Beats = %d, Duration = %lf us" %(reps, beats, usduration)) # for debug

            if usduration < fiveseconds:
                reps = reps*2

        dnsduration.append(usduration)
        dsduration.append(dnsduration[test]/1000000.0)
        dbytes.append(reps*beats*(typesize>>3))
        dmbytes.append(dbytes[test]/(1024 * 1024))
        bpersec.append(2.0*dbytes[test]/dsduration[test])
        mbpersec.append(2.0*bpersec[test]/(1024 * 1024))
        throughput.append(mbpersec[test])
        print("Test %d, Throughput: %d MB/s" %(test, throughput[test]))
        beats = beats*4
        test+=1

    #cleanup
    xrtBOFree(input_bo1)
    xrtBOFree(input_bo2)
    xrtBOFree(output_bo1)
    xrtBOFree(output_bo2)
    xrtKernelClose(khandle1)
    xrtKernelClose(khandle2)

    if failed:
        raise RuntimeError("ERROR: Failed to copy entries")

    print("TTTT: %d" %throughput[0])
    print("Maximum throughput: %d MB/s" %max(throughput))
    if max(throughput) < threshold:
        raise RuntimeError("ERROR: Throughput is less than expected value of %d GB/sec" %(threshold/1000))

def main(args):
    opt = Options()
    Options.getOptions(opt, args)

    try:
        initXRT(opt)
        assert (opt.first_mem >= 0), "Incorrect memory configuration"

        runKernel(opt)
        print("PASSED TEST")

    except OSError as o:
        print(o)
        print("FAILED TEST")
        sys.exit(o.errno)
    except AssertionError as a:
        print(a)
        print("FAILED TEST")
        sys.exit(1)
    except Exception as e:
        print(e)
        print("FAILED TEST")
        sys.exit(1)
    finally:
        xrtDeviceClose(opt.handle)

if __name__ == "__main__":
    main(sys.argv)



import pyopencl as cl
import numpy as np
import sys
from optparse import OptionParser
import time
import math

current_micro_time = lambda: int(round(time.time() * 1000000))

def cleanup(buffer_list):
   for buf in buffer_list:
      buf.release()

def main():
    platform_ID = None
    xclbin = None
    globalbuffersize = 1024*1024*0.07    #0.07 MB
    typesize = 512
    threshold = 40000
 
    # Process cmd line args
    parser = OptionParser()
    parser.add_option("-k", "--kernel", help="xclbin path")
    parser.add_option("-d", "--device", help="device index")
 
    (options, args) = parser.parse_args()
    xclbin = options.kernel
    index = options.device
    
    if xclbin is None:
       print("No xclbin specified\nUsage: -k <path to xclbin>")
       sys.exit(1)
    
    if index is None:
       index = 0 #get default device
    
    platforms = cl.get_platforms()
    # get Xilinx platform 
    for i in platforms:
       if i.name == "Xilinx":
          platform_ID = platforms.index(i)
          print("\nPlatform Information:")
          print("Platform name:       %s" %platforms[platform_ID].name)
          print("Platform version:    %s" %platforms[platform_ID].version)
          print("Platform profile:    %s" %platforms[platform_ID].profile)
          print("Platform extensions: %s" %platforms[platform_ID].extensions)
          break
 
    if platform_ID is None:
       #make sure xrt is sourced
       #run clinfo to make sure Xilinx platform is discoverable
       print("ERROR: Plaform not found")
       sys.exit(1)
 
    # choose device
    devices = platforms[platform_ID].get_devices()
    if int(index) > len(devices)-1:
       print("\nERROR: Index out of range. %d devices were found" % len(devices))
       sys.exit(1)
    else:
       dev = devices[int(index)]

    ctx = cl.Context(devices = [dev])
    if not ctx:
       print("ERROR: Failed to create context")
       sys.exit(1)
 
    commands = cl.CommandQueue(ctx, dev, properties=cl.command_queue_properties.OUT_OF_ORDER_EXEC_MODE_ENABLE)
 
    if not commands:
       print("ERROR: Failed to create command queue")
       sys.exit(1)
 
    print("Loading xclbin")
    with open(xclbin, "rb") as f:
       src = f.read()
    prg = cl.Program(ctx, [dev], [src])
    try:
       prg.build()
    except:
       print("ERROR:")
       print(prg.get_build_info(ctx, cl.program_build_info.LOG))
       raise
 
    knl1 = prg.bandwidth1
    knl2 = prg.bandwidth2
    
    #input host and buffer
    lst = [i%256 for i in range(int(globalbuffersize))]
    input_host1 = np.array(lst).astype(np.uint8)
    input_host2 = np.array(lst).astype(np.uint8)
    input_host3 = np.array(lst).astype(np.uint8)
    input_host4 = np.array(lst).astype(np.uint8)
    input_host5 = np.array(lst).astype(np.uint8)
    input_host6 = np.array(lst).astype(np.uint8)

    input_buf1 = cl.Buffer(ctx, cl.mem_flags.READ_WRITE | cl.mem_flags.COPY_HOST_PTR, hostbuf = input_host1)
    input_buf2 = cl.Buffer(ctx, cl.mem_flags.READ_WRITE | cl.mem_flags.COPY_HOST_PTR, hostbuf = input_host2)
    input_buf3 = cl.Buffer(ctx, cl.mem_flags.READ_WRITE | cl.mem_flags.COPY_HOST_PTR, hostbuf = input_host3)
    input_buf4 = cl.Buffer(ctx, cl.mem_flags.READ_WRITE | cl.mem_flags.COPY_HOST_PTR, hostbuf = input_host4)
    input_buf5 = cl.Buffer(ctx, cl.mem_flags.READ_WRITE | cl.mem_flags.COPY_HOST_PTR, hostbuf = input_host5)
    input_buf6 = cl.Buffer(ctx, cl.mem_flags.READ_WRITE | cl.mem_flags.COPY_HOST_PTR, hostbuf = input_host6)
    if any(buf is None for buf in [input_buf1.int_ptr, input_buf2.int_ptr, input_buf3.int_ptr, input_buf4.int_ptr, input_buf5.int_ptr, input_buf6.int_ptr]):
       print("ERROR: Failed to allocate source buffer")
       sys.exit(1)
    
    #output host and buffer
    output_host1 = np.empty_like(input_host1, dtype=np.uint8)
    output_host2 = np.empty_like(input_host2, dtype=np.uint8)
    output_host3 = np.empty_like(input_host2, dtype=np.uint8)
    output_host4 = np.empty_like(input_host2, dtype=np.uint8)
    output_host5 = np.empty_like(input_host2, dtype=np.uint8)
    output_host6 = np.empty_like(input_host2, dtype=np.uint8)
    
    output_buf1 = cl.Buffer(ctx, cl.mem_flags.READ_WRITE, output_host1.nbytes)
    output_buf2 = cl.Buffer(ctx, cl.mem_flags.READ_WRITE, output_host2.nbytes)
    output_buf3 = cl.Buffer(ctx, cl.mem_flags.READ_WRITE, output_host3.nbytes)
    output_buf4 = cl.Buffer(ctx, cl.mem_flags.READ_WRITE, output_host4.nbytes)
    output_buf5 = cl.Buffer(ctx, cl.mem_flags.READ_WRITE, output_host5.nbytes)
    output_buf6 = cl.Buffer(ctx, cl.mem_flags.READ_WRITE, output_host6.nbytes)
    
    if any(buf is None for buf in [output_buf1.int_ptr, output_buf2.int_ptr, output_buf3.int_ptr, output_buf4.int_ptr, output_buf5.int_ptr, output_buf6.int_ptr]):
       print("ERROR: Failed to allocate destination buffer")
       sys.exit(1)

    #copy dataset to OpenCL buffer
    globalbuffersizeinbeats = globalbuffersize/(typesize/8)
    tests= int(math.log(globalbuffersizeinbeats, 2.0))+1
 
    #lists
    buffer_list = [input_buf1, input_buf2, input_buf3, input_buf4, input_buf5, input_buf6, output_buf1, output_buf2, output_buf3, output_buf4, output_buf5, output_buf6]
    dnsduration = []
    dsduration  = []
    dbytes      = []
    dmbytes     = []
    bpersec     = []
    mbpersec    = []

    #run tests with burst length 1 beat to globalbuffersize
    #double burst length each test
    test=0
    beats = 16
    throughput = []
    while beats <= 1024:
        print("LOOP PIPELINE %d beats" % beats)
 
        usduration = 0
        fiveseconds = 5*1000000
        reps = 64
        while usduration < fiveseconds:

            start = current_micro_time()
            knl1(commands, (1, ), (1, ), output_buf1, input_buf1, output_buf2, input_buf2, output_buf3, input_buf3, np.uint32(beats), np.uint32(reps))
            knl2(commands, (1, ), (1, ), output_buf4, input_buf4, output_buf5, input_buf5, output_buf6, input_buf6, np.uint32(beats), np.uint32(reps))
            commands.finish()
            end = current_micro_time()

            usduration = end-start

            cl.enqueue_copy(commands, output_host1, output_buf1).wait()
            cl.enqueue_copy(commands, output_host2, output_buf2).wait()
            cl.enqueue_copy(commands, output_host3, output_buf3).wait()
            cl.enqueue_copy(commands, output_host4, output_buf4).wait()
            cl.enqueue_copy(commands, output_host5, output_buf5).wait()
            cl.enqueue_copy(commands, output_host6, output_buf6).wait()
            
            limit = int(beats*(typesize/8))
            if not np.array_equal(output_host1[:limit], input_host1[:limit]):
               print("ERROR: Failed to copy entries")
               cleanup(buffer_list)
               sys.exit(1)

            if not np.array_equal(output_host2[:limit], input_host2[:limit]):
               print("ERROR: Failed to copy entries")
               cleanup(buffer_list)
               sys.exit(1)
            
            if not np.array_equal(output_host3[:limit], input_host3[:limit]):
               print("ERROR: Failed to copy entries")
               cleanup(buffer_list)
               sys.exit(1)

            if not np.array_equal(output_host4[:limit], input_host4[:limit]):
               print("ERROR: Failed to copy entries")
               cleanup(buffer_list)
               sys.exit(1)

            if not np.array_equal(output_host5[:limit], input_host5[:limit]):
               print("ERROR: Failed to copy entries")
               cleanup(buffer_list)
               sys.exit(1)
            
            if not np.array_equal(output_host6[:limit], input_host6[:limit]):
               print("ERROR: Failed to copy entries")
               cleanup(buffer_list)
               sys.exit(1)
            
            # print("Reps = %d, Beats = %d, Duration = %lf us" %(reps, beats, usduration)) # for debug

            if usduration < fiveseconds:
                reps = reps*2


        dnsduration.append(usduration)
        dsduration.append(dnsduration[test]/1000000.0)
        dbytes.append(reps*beats*(typesize/8))
        dmbytes.append(dbytes[test]/(1024 * 1024))
        bpersec.append(6.0*dbytes[test]/dsduration[test])
        mbpersec.append(2.0*bpersec[test]/(1024 * 1024))
        throughput.append(mbpersec[test])
        print("Test %d, Throughput: %d MB/s" %(test, throughput[test]))
        beats = beats*4
        test+=1
    
    #cleanup
    cleanup(buffer_list)
    del ctx

    print("TTTT: %d" %throughput[0])
    print("Maximum throughput: %d MB/s" % max(throughput))
    if max(throughput) < threshold:
        print("ERROR: Throughput is less than expected value of %d GB/sec" %(threshold/1000))
        sys.exit(1)
    
    print("PASSED")

if __name__ == "__main__":
    main()



#!/usr/bin/python3

import sys
import uuid
from xrt_binding import * # found in PYTHONPATH
sys.path.append('../') # utils_binding.py
from utils_binding import *

XVECTORSWIZZLE_CONTROL_ADDR_AP_CTRL = 0x00
XVECTORSWIZZLE_CONTROL_ADDR_GIE = 0x04
XVECTORSWIZZLE_CONTROL_ADDR_IER = 0x08
XVECTORSWIZZLE_CONTROL_ADDR_ISR = 0x0c
XVECTORSWIZZLE_CONTROL_ADDR_GROUP_ID_X_DATA = 0x10
XVECTORSWIZZLE_CONTROL_BITS_GROUP_ID_X_DATA = 32
XVECTORSWIZZLE_CONTROL_ADDR_GROUP_ID_Y_DATA = 0x18
XVECTORSWIZZLE_CONTROL_BITS_GROUP_ID_Y_DATA = 32
XVECTORSWIZZLE_CONTROL_ADDR_GROUP_ID_Z_DATA = 0x20
XVECTORSWIZZLE_CONTROL_BITS_GROUP_ID_Z_DATA = 32
XVECTORSWIZZLE_CONTROL_ADDR_GLOBAL_OFFSET_X_DATA = 0x28
XVECTORSWIZZLE_CONTROL_BITS_GLOBAL_OFFSET_X_DATA = 32
XVECTORSWIZZLE_CONTROL_ADDR_GLOBAL_OFFSET_Y_DATA = 0x30
XVECTORSWIZZLE_CONTROL_BITS_GLOBAL_OFFSET_Y_DATA = 32
XVECTORSWIZZLE_CONTROL_ADDR_GLOBAL_OFFSET_Z_DATA = 0x38
XVECTORSWIZZLE_CONTROL_BITS_GLOBAL_OFFSET_Z_DATA = 32
XVECTORSWIZZLE_CONTROL_ADDR_A_DATA = 0x40

def runKernel(opt):
    elem_num = 4096
    DATA_SIZE = ctypes.sizeof(ctypes.c_int) * elem_num
    boHandle = xclAllocBO(opt.handle, DATA_SIZE, 0, opt.first_mem)
    bo = xclMapBO(opt.handle, boHandle, True)
    ctypes.memset(bo, 0, DATA_SIZE)
    bo_arr = ctypes.cast(bo, ctypes.POINTER(ctypes.c_int))
    reference = []

    for idx in range(elem_num):
        remainder = idx % 4
        bo_arr[idx] = idx
        if remainder == 0:
            reference.append(idx+2)
        if remainder == 1:
            reference.append(idx+2)
        if remainder == 2:
            reference.append(idx-2)
        if remainder == 3:
            reference.append(idx-2)

    if xclSyncBO(opt.handle, boHandle, xclBOSyncDirection.XCL_BO_SYNC_BO_TO_DEVICE, DATA_SIZE, 0):
        return 1

    p = xclBOProperties()
    bodevAddr = p.paddr if not (xclGetBOProperties(opt.handle, boHandle, p)) else -1

    if bodevAddr is -1:
        return 1

    # Allocate the exec_bo
    execHandle = xclAllocBO(opt.handle, DATA_SIZE, 0, (1 << 31))
    execData = xclMapBO(opt.handle, execHandle, True)  # returns mmap()

    print("Construct the exec command to run the kernel on FPGA")

    xclOpenContext(opt.handle, opt.xuuid, 0, True)
    # construct the exec buffer cmd to start the kernel
    start_cmd = ert_start_kernel_cmd.from_buffer(execData.contents)
    rsz = (XVECTORSWIZZLE_CONTROL_ADDR_A_DATA / 4 + 1) + 1  # regmap array size
    ctypes.memset(execData.contents, 0, ctypes.sizeof(ert_start_kernel_cmd) + rsz*4)
    start_cmd.m_uert.m_start_cmd_struct.state = 1  # ERT_CMD_STATE_NEW
    start_cmd.m_uert.m_start_cmd_struct.opcode = 0  # ERT_START_CU
    start_cmd.m_uert.m_start_cmd_struct.count = 1 + rsz
    start_cmd.cu_mask = 0x1

    # Prepare kernel reg map
    new_data = (ctypes.c_uint32 * rsz).from_buffer(execData.contents, 8)
    new_data[XVECTORSWIZZLE_CONTROL_ADDR_AP_CTRL] = 0x0 # ap_start
    new_data[XVECTORSWIZZLE_CONTROL_ADDR_GROUP_ID_X_DATA/4] = 0x0 # group id
    new_data[XVECTORSWIZZLE_CONTROL_ADDR_A_DATA / 4] = bodevAddr
    new_data[XVECTORSWIZZLE_CONTROL_ADDR_A_DATA / 4 + 1] = (bodevAddr >> 32) & 0xFFFFFFFF # s1 buffer

    global_dim = [DATA_SIZE / 4, 0]; # int4 vector count global range
    local_dim = [16, 0]; # int4 vector count global range
    groupSize = global_dim[0] / local_dim[0];

    if opt.verbose == 1:
        print("Global range: %d " % global_dim[0])
        print("Group size  : %d " % local_dim[0])
        print("Starting kernel...\n")


    for id in range(groupSize):
        if opt.verbose == 1:
            print("group id = %d" % id)

        new_data[XVECTORSWIZZLE_CONTROL_ADDR_AP_CTRL] = 0x0 # ap_start
        new_data[XVECTORSWIZZLE_CONTROL_ADDR_GROUP_ID_X_DATA/4] = id # group id

        # Execute the command
        if xclExecBuf(opt.handle, execHandle):
            print("Unable to issue xclExecBuf")
            return 1

        if opt.verbose == 1:
            print("Wait until the command finish")

        while xclExecWait(opt.handle, 1000) == 0:
            print(".")

    # get the output xclSyncBO
    print("Get the output data from the device")
    if xclSyncBO(opt.handle, boHandle, xclBOSyncDirection.XCL_BO_SYNC_BO_FROM_DEVICE, DATA_SIZE, 0):
        return 1

    fail = 0
    print("Compare the FPGA results with golden data")
    for idx in range(elem_num):
        if bo_arr[idx] != reference[idx]:
            fail = 1
            print("FAIL: Results mismatch at [%d] fpga= %d, cpu= %d" % (idx, bo_arr[idx], reference[idx]))
            break

    xclCloseContext(opt.handle, opt.xuuid, 0)
    xclFreeBO(opt.handle, execHandle)
    xclFreeBO(opt.handle, boHandle)

    return fail

def main(args):
    opt = Options()
    Options.getOptions(opt, args)

    try:
        if initXRT(opt):
            xclClose(opt.handle)
            return 1
        if opt.first_mem < 0:
            xclClose(opt.handle)
            return 1
        if runKernel(opt):
            xclClose(opt.handle)
            return 1

    except Exception as exp:
        print("Exception: ")
        print(exp)  # prints the err
        print("FAILED TEST")
        sys.exit()

    print("PASSED TEST")


if __name__ == "__main__":
    main(sys.argv)



#!/usr/bin/python3

"""
 Copyright (C) 2019-2020 Xilinx, Inc

 ctypes based Python binding for XRT

 Licensed under the Apache License, Version 2.0 (the "License"). You may
 not use this file except in compliance with the License. A copy of the
 License is located at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 License for the specific language governing permissions and limitations
 under the License.
"""

import sys
# Following is found in PYTHONPATH
from xrt_binding import *
# Following is found in ..
sys.path.append('../')
from utils_binding import *

def main(args):
    opt = Options()
    Options.getOptions(opt, args)
    try:
        initXRT(opt)
        assert (opt.first_mem >= 0), "Incorrect memory configuration"

        boHandle1 = xclAllocBO(opt.handle, opt.DATA_SIZE, 0, opt.first_mem)
        bo1 = xclMapBO(opt.handle, boHandle1, True)

        testVector = "hello\nthis is Xilinx OpenCL memory read write test\n:-)\n"
        ctypes.memset(bo1, 0, opt.DATA_SIZE)
        ctypes.memmove(bo1, testVector, len(testVector))

        xclSyncBO(opt.handle, boHandle1, xclBOSyncDirection.XCL_BO_SYNC_BO_TO_DEVICE, opt.DATA_SIZE, 0)

        p = xclBOProperties(boHandle1)
        xclGetBOProperties(opt.handle, boHandle1, p)
        assert (p.paddr != 0xffffffffffffffff), "Illegal physical address for buffer"

        # Clear our shadow buffer on host
        ctypes.memset(bo1, 0, opt.DATA_SIZE)

        # Move the buffer from device back to the shadow buffer on host
        xclSyncBO(opt.handle, boHandle1, xclBOSyncDirection.XCL_BO_SYNC_BO_FROM_DEVICE, opt.DATA_SIZE, False)

        assert (bo1[:len(testVector)] == testVector[:]), "Data migration error"
        print("PASSED TEST")
        xclUnmapBO(opt.handle, boHandle1, bo1)
        xclFreeBO(opt.handle, boHandle1)

    except OSError as o:
        print(o)
        print("FAILED TEST")
        sys.exit(o.errno)
    except AssertionError as a:
        print(a)
        print("FAILED TEST")
        sys.exit(1)
    except Exception as e:
        print(e)
        print("FAILED TEST")
        sys.exit(1)
    finally:
        xclClose(opt.handle)

if __name__ == "__main__":
    main(sys.argv)



"""
 Copyright (C) 2018 Xilinx, Inc
 Author(s): Ryan Radjabi
            Shivangi Agarwal
            Sonal Santan
 ctypes based Python binding for xclbin.h data structures

 Licensed under the Apache License, Version 2.0 (the "License"). You may
 not use this file except in compliance with the License. A copy of the
 License is located at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 License for the specific language governing permissions and limitations
 under the License.
"""

import os
import ctypes

libc = ctypes.CDLL(os.environ['XILINX_XRT'] + "/lib/libxrt_core.so")


"""
    AXLF LAYOUT
    -----------

    -----------------------------------------
    | Magic                                 |
    -----------------------------------------
    | Header                                |
    -----------------------------------------
    | One or more section headers           |
    -----------------------------------------
    | Matching number of sections with data |
    -----------------------------------------
"""


class AXLF_SECTION_KIND:
    BITSTREAM             = 0
    CLEARING_BITSTREAM    = 1
    EMBEDDED_METADATA     = 2
    FIRMWARE              = 3
    DEBUG_DATA            = 4
    SCHED_FIRMWARE        = 5
    MEM_TOPOLOGY          = 6
    CONNECTIVITY          = 7
    IP_LAYOUT             = 8
    DEBUG_IP_LAYOUT       = 9
    DESIGN_CHECK_POINT    = 10
    CLOCK_FREQ_TOPOLOGY   = 11
    MCS                   = 12
    BMC                   = 13
    BUILD_METADATA        = 14
    KEYVALUE_METADATA     = 15
    USER_METADATA         = 16
    DNA_CERTIFICATE       = 17
    PDI                   = 18
    BITSTREAM_PARTIAL_PDI = 19
    PARTITION_METADATA    = 20
    EMULATION_DATA        = 21
    SYSTEM_METADATA       = 22

class MEM_TYPE:
    MEM_DDR3                 = 0
    MEM_DDR4                 = 1
    MEM_DRAM                 = 2
    MEM_STREAMING            = 3
    MEM_PREALLOCATED_GLOB    = 4
    MEM_ARE                  = 5
    MEM_HBM                  = 6
    MEM_BRAM                 = 7
    MEM_URAM                 = 8
    MEM_STREAMING_CONNECTION = 9

class IP_TYPE:
    IP_MB              = 0
    IP_KERNEL          = 1
    IP_DNASC           = 2
    IP_DDR4_CONTROLLER = 3
    IP_MEM_DDR4        = 4
    IP_MEM_HBM         = 5

class XCLBIN_MODE:
    XCLBIN_FLAT                  = 1
    XCLBIN_PR                    = 2
    XCLBIN_TANDEM_STAGE2         = 3
    XCLBIN_TANDEM_STAGE2_WITH_PR = 4
    XCLBIN_HW_EMU                = 5
    XCLBIN_SW_EMU                = 6
    XCLBIN_MODE_MAX              = 7

class axlf_section_header (ctypes.Structure):
    _fields_ = [
        ("m_sectionKind", ctypes.c_uint32),
        ("m_sectionName", ctypes.c_char*16),
        ("m_sectionOffset", ctypes.c_uint64),
        ("m_sectionSize", ctypes.c_uint64)
    ]

class s1 (ctypes.Structure):
    _fields_ = [
        ("m_platformId", ctypes.c_uint64),
        ("m_featureId", ctypes.c_uint64)
    ]

class u1 (ctypes.Union):
    _fields_ = [
        ("rom", s1),
        ("rom_uuid", ctypes.c_ubyte*16)
    ]

class u2 (ctypes.Union):
    _fields_ = [
        ("m_next_axlf", ctypes.c_char*16),
        ("uuid", ctypes.c_ubyte*16)  # uuid_t/xuid_t
    ]

class axlf_header (ctypes.Structure):
    _anonymous_ = ("u1","u2")
    _fields_ = [
        ("m_length", ctypes.c_uint64),
        ("m_timeStamp", ctypes.c_uint64),
        ("m_featureRomTimeStamp", ctypes.c_uint64),
        ("m_versionPatch", ctypes.c_uint16),
        ("m_versionMajor", ctypes.c_uint8),
        ("m_versionMinor", ctypes.c_uint8),
        ("m_mode", ctypes.c_uint32),
        ("u1", u1),
        ("m_platformVBNV", ctypes.c_ubyte*64),
        ("u2", u2),
        ("m_debug_bin", ctypes.c_char*16),
        ("m_numSections", ctypes.c_uint32)
    ]

class axlf (ctypes.Structure):
    _fields_ = [
        ("m_magic", ctypes.c_char*8),
        ("m_cipher", ctypes.c_ubyte*32),
        ("m_keyBlock", ctypes.c_ubyte*256),
        ("m_uniqueId", ctypes.c_uint64),
        ("m_header", axlf_header),
        ("m_sections", axlf_section_header)
    ]

class xlnx_bitstream(ctypes.Structure):
    _fields_ = [
        ("m_freq", ctypes.c_uint8*8),
        ("bits", ctypes.c_char*1)
    ]


"""   MEMORY TOPOLOGY SECTION   """

class mem_u1 (ctypes.Union):
    _fields_ = [
        ("m_size", ctypes.c_int64),
        ("route_id", ctypes.c_int64)
    ]

class mem_u2 (ctypes.Union):
    _fields_ = [
        ("m_base_address", ctypes.c_int64),
        ("flow_id", ctypes.c_int64)
    ]

class mem_data (ctypes.Structure):
    _anonymous_ = ("mem_u1", "mem_u2")
    _fields_ = [
        ("m_type", ctypes.c_uint8),
        ("m_used", ctypes.c_uint8),
        ("mem_u1", mem_u1),
        ("mem_u2", mem_u2),
        ("m_tag", ctypes.c_char * 16)
    ]

class mem_topology (ctypes.Structure):
    _fields_ = [
        ("m_count", ctypes.c_int32),
        ("m_mem_data", mem_data*1)
    ]


"""   CONNECTIVITY SECTION   """

class connection(ctypes.Structure):
    _fields_ = [
        ("arg_index", ctypes.c_int32),
        ("m_ip_layout_index", ctypes.c_int32),
        ("mem_data_index", ctypes.c_int32)
    ]

class connectivity(ctypes.Structure):
    _fields_ = [
        ("m_count", ctypes.c_int32),
        ("m_connection", connection*1)
    ]


"""   IP_LAYOUT SECTION   """

class IP_CONTROL:
    AP_CTRL_HS    = 0
    AP_CTRL_CHAIN = 1
    AP_CTRL_NONE  = 2
    AP_CTRL_ME    = 3

class indices (ctypes.Structure):
    _fields_ = [
        ("m_index", ctypes.c_uint16),
        ("m_pc_index", ctypes.c_uint8),
        ("unused", ctypes.c_uint8)
    ]

class ip_u1 (ctypes.Union):
    _fields_ = [
        ("m_base_address", ctypes.c_int64),
        ("indices", indices)
    ]

class ip_data (ctypes.Structure):
    _fields_ = [
        ("m_type", ctypes.c_uint32),
        ("properties", ctypes.c_uint32),
        ("ip_u1", ip_u1),
        ("m_name", ctypes.c_uint8 * 64)
    ]

class ip_layout (ctypes.Structure):
    _fields_ = [
        ("m_count", ctypes.c_int32),
        ("m_ip_data", ip_data*1)
    ]


""" Debug IP section layout """

class DEBUG_IP_TYPE:
    UNDEFINED                   = 0
    LAPC                        = 1
    ILA                         = 2
    AXI_MM_MONITOR              = 3
    AXI_TRACE_FUNNEL            = 4
    AXI_MONITOR_FIFO_LITE       = 5
    AXI_MONITOR_FIFO_FULL       = 6
    ACCEL_MONITOR               = 7
    AXI_STREAM_MONITOR          = 8
    AXI_STREAM_MONITOR          = 9
    AXI_STREAM_PROTOCOL_CHECKER = 10
    TRACE_S2MM                  = 11
    AXI_DMA                     = 12

class debug_ip_data(ctypes.Structure):
    _fields_ = [
        ("m_type", ctypes.c_uint8),
        ("m_index", ctypes.c_uint8),
        ("m_properties", ctypes.c_uint8),
        ("m_major", ctypes.c_uint8),
        ("m_minor", ctypes.c_uint8),
        ("m_reserved", ctypes.c_uint8*3),
        ("m_base_address", ctypes.c_uint64),
        ("m_name", ctypes.c_uint8*128)
    ]

class debug_ip_layout(ctypes.Structure):
    _fields_ = [
        ("m_count", ctypes.c_uint16),
        ("m_debug_ip_data", debug_ip_data*1)
    ]

class CLOCK_TYPE:
    CT_UNUSED = 0
    CT_DATA   = 1
    CT_KERNEL = 2
    CT_SYSTEM = 3

class clock_freq(ctypes.Structure):
    _fields_ = [
        ("m_freq_Mhz", ctypes.c_int16),
        ("m_type", ctypes.c_int8),
        ("m_unused", ctypes.c_uint8*5),
        ("m_name", ctypes.c_char*128)
    ]

class clock_freq_topology(ctypes.Structure):
    _fields_ = [
        ("m_count", ctypes.c_uint16),
        ("m_clock_freq", clock_freq*1)
    ]

class MCS_TYPE:
    MCS_UNKNOWN   = 0
    MCS_PRIMARY   = 1
    MCS_SECONDARY = 2

class mcs_chunk(ctypes.Structure):
    _fields_ = [
        ("m_type", ctypes.c_uint8),
        ("m_unused", ctypes.c_uint8*7),
        ("m_offset", ctypes.c_uint64),
        ("m_size", ctypes.c_uint64)
    ]

class mcs(ctypes.Structure):
    _fields_ = [
        ("m_count", ctypes.c_int8),
        ("m_unused", ctypes.c_int8),
        ("m_chunk", mcs_chunk*1)
    ]

class bmc(ctypes.Structure):
    _fields_ = [
        ("m_offset", ctypes.c_uint64),
        ("m_size", ctypes.c_uint64),
        ("m_image_name", ctypes.c_char*64),
        ("m_device_name", ctypes.c_char*64),
        ("m_version", ctypes.c_char*64),
        ("m_md5value", ctypes.c_char*33),
        ("m_padding", ctypes.c_char*7)
    ]

class CHECKSUM_TYPE:
    CST_UNKNOWN = 0
    CST_SDBM    = 1
    CST_LAST    = 2

def wrap_get_axlf_section(top, kind):
    libc.wrap_get_axlf_section.restype = ctypes.POINTER(axlf_section_header)
    libc.wrap_get_axlf_section.argtypes = [ctypes.c_void_p, ctypes.c_int]
    return libc.wrap_get_axlf_section(top, kind)



#!/usr/bin/env python3
"""
 Copyright (C) 2020 Xilinx, Inc
 Author(s): Brian Xu (brianx@xlinx.com)

 Licensed under the Apache License, Version 2.0 (the "License"). You may
 not use this file except in compliance with the License. A copy of the
 License is located at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 License for the specific language governing permissions and limitations
 under the License.
"""

import sys
import argparse
import textwrap
import os
import re
import subprocess
import time

#In this 'lspci -t' snippet
#-+-[0000:16]-+-00.0-[17]--+-00.0
# |           |            \-00.1
# |           +-01.0-[18]--+-00.0
# |           |            \-00.1
#
#FPGA 17:00.[01] parent is 16.00.0, which has primary bus number 16, and secondary bus number 17
#FPGA 18:00.[01] parent is 16.01.0, which has primary bus number 16, and secondary bus number 18
#FPGAs whose parents have identical pcie primary bus number are on same card

#FPGA node and its parent primary bus number mapping
#eg "0000:17:00.0" "16"
#   "0000:18:00.0" "16"
node_bus_mapping = {}
#FPGA node and its parent node DBDF mapping
#eg "0000:17:00.0" "0000:16:00.0"
#   "0000:18:00.0" "0000:16:01.0"
node_parent_mapping = {}
#primary bus of pcie bridge and FPGA node mapping
#eg "16" ["0000:17.00.0", "0000:18:00.0"]
bus_children = {}

#path of the ps_ready sysfs node of the FPGA
ps_ready = {}

rootDir = "/sys/bus/pci/devices/"

#get all above "node vs bus" mappings of xilinx FPGAs 
def get_node_bus_mapping():
    subdir = os.listdir(rootDir)
    for subdirName in subdir:
        if re.search("^[0-9a-fA-F]{4}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}.0$", subdirName) == None:
            continue
        with open(os.path.join(rootDir, subdirName, "vendor")) as f : vendor = f.read()
        if vendor.strip() != "0x10ee":
            continue
        files = os.listdir(os.path.join(rootDir, subdirName))
        for fname in files:
            if re.search("^processor_system.+$", fname) != None:
                ps_ready[subdirName] = os.path.join(rootDir, subdirName, fname, "ps_ready")
                break
        #print("%s: %s " % (subdirName, ps_ready[subdirName]))
        files = os.listdir(os.path.join(rootDir, subdirName, "dparent"))
        for fname in files:
            if re.search("^[0-9a-fA-F]{4}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}.[0-7]:pcie.+$", fname) != None:
            	#print("\tfile: %s" % fname)
                bus = fname.split(":")[1]
                node_bus_mapping[subdirName] = bus
                node_parent_mapping[subdirName] = fname[:fname.rfind(":")]
                if bus not in bus_children:
                    bus_children[bus] = []
                bus_children[bus].append(subdirName);

#do pcie node remove and rescan
def run(cmdline):
    #print(cmdline)
    subprocess.call(cmdline, shell=True)

#wait ps on FPGA back online
def wait_ps_online(mgmt):
    while True:
        with open(ps_ready[mgmt]) as f : ready = f.read()
        if ready.strip() == '0':
            time.sleep(3)
            continue
        print("%s ps ready" % mgmt)
        break

#wait FPGA back online
def wait_xocl_online(user):
    while True:
        with open(os.path.join(rootDir, user, "dev_offline")) as f : ready = f.read()
        if ready.strip() == '1':
            time.sleep(1)
            continue
        print("%s host online" % user)
        break

def main():
    desc = textwrap.dedent('''\
        This cmdline tool is mainly used to do card level reset.
        Eg. U30 card which has 2 FPGAs in one card. When whichever FPGA is
        specified, both FPGAs on the card will be reset.

        Root privilege is required to run this cmd
        
        Example:
            xbrest -d 0000:17:0.1
    ''')
    if (os.geteuid() != 0):
        print("ERROR: root privilege is required.")
        sys.exit(1)

    parser=argparse.ArgumentParser(description=desc,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument("-d","--dbdf", required=True,
        help="DBDF of a FPGA node, with format xxxx:xx:xx.x")
    parser.add_argument("-y","--yes", help="automatic yes to prompts",
        action="store_true")
    args = parser.parse_args()

    get_node_bus_mapping()
    mgmt = args.dbdf[:len(args.dbdf)-1] + "0"
    user = args.dbdf[:len(args.dbdf)-1] + "1"

    try:
        if mgmt not in node_bus_mapping:
            print("ERROR: Is %s a FPGA node?" % user)
            sys.exit(1)
        with open(os.path.join(rootDir, user, "ready")) as f : ready = f.read()
        ready = ready.strip("\n")
        if ready == "0x0" or ready == "0":
            print("ERROR: FPGA %s is not usable" % user)
            sys.exit(1)

        #print(node_parent_mapping)
        nodes_in_cards = bus_children[node_bus_mapping[mgmt]]
        #print(nodes_in_cards)
        print("Card level reset. This will reset all FPGAs on the card")
        print("All existing processes will be killed.")
        while not args.yes:
            line = input("Are you sure you wish to proceed? [y/n]:")
            if line == "n":
                sys.exit(1)
            if line == "y":
                break
        #steps to do card level reset
        #1. kill processes running on both cards
        #2. trigger SBR
        #3. remove nodes of that FPGA
        #4. rescan pcie 
        #5. wait back online
        for node in nodes_in_cards:
            user = node[:len(node)-1] + "1"
            print("shutdown: %s" % user)
            run("echo 2 > " + os.path.join(rootDir, user, "shutdown"))
        print("SBR reset...")
        run("echo 1 > " + os.path.join(rootDir, mgmt, "sbr_toggle"))
        time.sleep(3)
        for node in nodes_in_cards:
            user = node[:len(node)-1] + "1"
            print("remove: %s, %s" % (user, node))
            run("echo 1 > " + os.path.join(rootDir, user, "remove"))
            run("echo 1 > " + os.path.join(rootDir, node, "remove"))
        print("rescan pci")
        run("echo 1 > /sys/bus/pci/rescan")
        time.sleep(1)
        print("wait FPGAs back on line")
        for node in nodes_in_cards:
            user = node[:len(node)-1] + "1"
            wait_ps_online(node)
            wait_xocl_online(user)

    except (subprocess.TimeoutExpired):
        #xbutil reset timeout, try remove it
        print("shutdown: %s" % user)
        run("echo 2 > " + os.path.join(rootDir, user, "shutdown"))
        print("remove: %s, %s" % (user, mgmt))
        run("echo 1 > " + os.path.join(rootDir, node_parent_mapping[mgmt], "remove"))
        print("rescan pci")
        run("echo 1 > /sys/bus/pci/rescan")
    except Exception as e:
        print(e)

if __name__ == "__main__":
    main()



"""
 Copyright (C) 2019-2020 Xilinx, Inc

 ctypes based Python binding for XRT

 Licensed under the Apache License, Version 2.0 (the "License"). You may
 not use this file except in compliance with the License. A copy of the
 License is located at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 License for the specific language governing permissions and limitations
 under the License.
"""

import os
import sys
import uuid
import re

# Following found in PYTHONPATH setup by XRT
from xrt_binding import *
from ert_binding import *

# utils_binding.py
sys.path.append('../')
from utils_binding import *


def runKernel(opt):

    rule = re.compile("hello*")
    name = list(filter(lambda val: rule.match, opt.kernels))[0]
    khandle = xrtPLKernelOpen(opt.handle, opt.xuuid, name)

    grpid = xrtKernelArgGroupId(khandle, 0)
    if grpid < 0:
        raise RuntimeError("failed to find BO group ID: %d" % grpid)

    boHandle1 = xrtBOAlloc(opt.handle, opt.DATA_SIZE, 0, grpid)
    buf1 = xrtBOMap(boHandle1)
    bo1 = ctypes.cast(buf1, ctypes.POINTER(ctypes.c_char))
    if bo1 == 0:
        raise RuntimeError("failed to map buffer1")
    ctypes.memset(bo1, 0, opt.DATA_SIZE)

    boHandle2 = xrtBOAlloc(opt.handle, opt.DATA_SIZE, 0, grpid)
    buf2 = xrtBOMap(boHandle2)
    bo2 = ctypes.cast(buf2, ctypes.POINTER(ctypes.c_char))
    if bo2 == 0:
        raise RuntimeError("failed to map buffer2")
    ctypes.memset(bo2, 0, opt.DATA_SIZE)

    xrtBOSync(boHandle1, xclBOSyncDirection.XCL_BO_SYNC_BO_TO_DEVICE, opt.DATA_SIZE, 0)
    xrtBOSync(boHandle2, xclBOSyncDirection.XCL_BO_SYNC_BO_TO_DEVICE, opt.DATA_SIZE, 0)

    print("Original string = [%s]" % bo1[:64].decode("utf-8"))
    print("Original string = [%s]" % bo2[:64].decode("utf-8"))

    print("Issue kernel start requests")
    kfunc = xrtKernelGetFunc(xrtBufferHandle)
    rhandle1 = kfunc(khandle, boHandle1)
    rhandle2 = kfunc(khandle, boHandle2)

    print("Now wait for the kernels to finish using xrtRunWait()")
    xrtRunWait(rhandle1)
    xrtRunWait(rhandle2)

    print("Get the output data produced by the 2 kernel runs from the device")
    xrtBOSync(boHandle1, xclBOSyncDirection.XCL_BO_SYNC_BO_FROM_DEVICE, opt.DATA_SIZE, 0)
    xrtBOSync(boHandle2, xclBOSyncDirection.XCL_BO_SYNC_BO_FROM_DEVICE, opt.DATA_SIZE, 0)
    result1 = bo1[:len("Hello World")]
    result2 = bo2[:len("Hello World")]
    print("Result string = [%s]" % result1.decode("utf-8"))
    print("Result string = [%s]" % result2.decode("utf-8"))
    assert(result1.decode("utf-8") == "Hello World"), "Incorrect output from kernel"
    assert(result2.decode("utf-8") == "Hello World"), "Incorrect output from kernel"

    xrtRunClose(rhandle2)
    xrtRunClose(rhandle1)
    xrtKernelClose(khandle)
    xrtBOFree(boHandle2)
    xrtBOFree(boHandle1)

def main(args):
    opt = Options()
    Options.getOptions(opt, args)

    try:
        initXRT(opt)
        assert (opt.first_mem >= 0), "Incorrect memory configuration"

        runKernel(opt)
        print("PASSED TEST")

    except OSError as o:
        print(o)
        print("FAILED TEST")
        sys.exit(o.errno)
    except AssertionError as a:
        print(a)
        print("FAILED TEST")
        sys.exit(1)
    except Exception as e:
        print(e)
        print("FAILED TEST")
        sys.exit(1)
    finally:
        xrtDeviceClose(opt.handle)

if __name__ == "__main__":
    #os.environ["Runtime.xrt_bo"] = "false"
    main(sys.argv)



#!/usr/bin/python

import re
# import numpy as np

maxFreq = 500.0
inputFreq = 250.0
minVCOFreq = 800.0
maxVCOFreq = 1600.0

step = 100.0

table = {}
#config reg 0 : 7:0 : div, 15:8 mult, 25:16: frac mult

for vco in range(int(minVCOFreq), int(maxVCOFreq + 1.0), int(step)):
    print "================"
    vco = float(vco)
    print vco
    config0 = 1
    mul = vco/inputFreq;
    print mul;
    config0 = config0 | ((int)(mul) << 8)
    print format(config0, '08x')

    mul = round(mul, 3)
    frac = re.findall('\d+\.(\d+)', str(mul))
    print "frac: ", frac[0]
    if (int(frac[0]) > 0):
      config0 = config0 | (int(frac[0]) << 16)
      print format(config0, '08x')
      config0 = config0 | (0x00000001 << 26)
      print format(config0, '08x')


    for config1 in range(2,11):
        freq = vco/config1
        if (freq > maxFreq):
            continue
        print "Freq: %s, config0: %08x, config1: %08x" %(freq, config0, config1)
        table[int(freq)] = [int(config0), int(config1), int(vco)]
    print "================"

# for freq in sorted(table.keys()):
#     #print (freq, '--->', "08x"%(int(table[freq][0])))
#     print ("\t%u, \t0x%08x, \t0x%08x, \t%u " %(freq, table[freq][0], table[freq][1], table[freq][2]))

print "================"
for freq,v in sorted(table.items()):
    #print (freq, '--->', "08x"%(int(table[freq][0])))
    print ("\t%u, \t0x%08x, \t0x%08x, \t%u " %(freq, v[0], v[1], v[2]))

prevFreq = 10
print "================"
for freq,v in sorted(table.items()):
    if ((freq - prevFreq) < 5):
        continue
    prevFreq = freq
    #print (freq, '--->', "08x"%(int(table[freq][0])))
    print ("\t{/*%u*/\t%u,\t0x%08x,\t0x%08x}," %(v[2], freq, v[0], v[1]))



#!/usr/bin/python

import os, sys, getopt
import subprocess, shlex
import glob, re

def create_xclbin1(filelist, outputfile):
    if(outputfile == ''):
	outputfile = "new.xclbin1"

    meta_file = '';
    delete_list = []
    cmd = ['xclbincat']

    for file in filelist:
	if file == "python-meta.xml":
	    print "meta file " + file
	    meta_file = file;
	    delete_list.append(file)
	elif file == "python-primary.bit":
	    print "primary file " + file
	    cmd.append("-bitstream")
	    cmd.append(file)
	    delete_list.append(file)
	elif file == "python-secondary.bit":
	    print "secondary file " + file
	    cmd.append("-clearstream")
	    cmd.append(file)
	    delete_list.append(file)
    if(meta_file != ''):
	cmd.append(meta_file)
    cmd.append(outputfile)
    cmd.append("-xclbin1");
    print "COMMAND: " + ' '.join(cmd)
    popen = subprocess.Popen(cmd, stdout=subprocess.PIPE);
    popen.wait()
    output = popen.stdout.read()
    print output
    for f in delete_list:
	os.remove(f)
    
def main(argv):
   inputfile = ''
   outputfile = ''
   try:
      opts, args = getopt.getopt(argv,"hi:o:",["ifile=","ofile="])
   except getopt.GetoptError:
      print 'test.py -i <inputfile> -o <outputfile>'
      sys.exit(2)
   for opt, arg in opts:
      if opt == '-h':
         print 'test.py -i <inputfile> -o <outputfile>'
         sys.exit()
      elif opt in ("-i", "--ifile"):
         inputfile = arg
      elif opt in ("-o", "--ofile"):
         outputfile = arg
   print 'Input file is :', inputfile
   print 'Output file is :', outputfile
   cmd = ['xclbinsplit']
   cmd.append(inputfile)
   cmd.append('-o')
   cmd.append('python')
   print "COMMAND: " + ' '.join(cmd)
   #popen = subprocess.Popen(['xclbinsplit', inputfile, '-o', 'python'], stdout=subprocess.PIPE);
   popen = subprocess.Popen(cmd, stdout=subprocess.PIPE);
   popen.wait()
   output = popen.stdout.read()
   print output
   x = glob.glob("python-*")
   create_xclbin1(x,outputfile)


if __name__ == "__main__":
   main(sys.argv[1:])



# =============================================================================
# Checks and validates the windows installation environment
# =============================================================================
#
# How to use this script.
# 1) Create an "x64 Native Tools compmand Prompt" via the Start menues
# 2) Invoke this script: xrtdeps-win.py
#
#
# Details
# -------
# The following directory structure will be produced if all of the options
# are used:
#    C:\Xilinx
#       +--XRT
#          +--ext
#             +--bin
#             +--build
#             |  +--boost
#             |  +--OpenCL-Headers
#             |  +--OpenCL-ICD-Loader
#             +--include
#             |  +--boost-1_70
#             |  +--CL
#             +--lib
# =============================================================================

# -- Imports ------------------------------------------------------------------
import argparse
from argparse import RawDescriptionHelpFormatter
import os
import platform
import subprocess
import pathlib
import distutils
from distutils import dir_util
import glob
import shutil
from shutil import copyfile
import urllib.request 


# -- Global Variables --------------------------------------------------------
COMPILER = "Visual Studio 15 2017 Win64"

if platform.system() == 'Windows':
  XRT_LIBRARY_INSTALL_DIR = "c:/Xilinx/XRT/ext"
  XRT_LIBRARY_BUILD_DIR = "c:/Xilinx/XRT/ext/build/"
else:
  XRT_LIBRARY_INSTALL_DIR = "/mnt/c/Xilinx/XRT/ext"
  XRT_LIBRARY_BUILD_DIR = "/mnt/c/Xilinx/XRT/ext/build"

# -- main() -------------------------------------------------------------------
#
# The entry point to this script.
#
# Note: It is called at the end of this script so that the other functions
#       and classes have been defined.
#
def main():
  # -- Configure the argument parser
  parser = argparse.ArgumentParser(formatter_class=RawDescriptionHelpFormatter, description='description:\n  validates the XRT Windows build environment\n  gets and installs XRT supporting libraries')
  parser.add_argument('--boost', nargs='?', const='complete', default='skip', choices=['complete', 'minimal', 'skip'], help='install boost libraries')
  parser.add_argument('--icd', action="store_true", help='install Khronos OpenCL icd loader library')
  parser.add_argument('--opencl', action="store_true", help='install Khronos OpenCL header files')
  parser.add_argument('--gtest', action="store_true", help='install gtest libraries')
  parser.add_argument('--validate_all_requirements', action="store_true", help='validate all XRT dependent libraries and tools are installed')
  parser.add_argument('--verbose', action="store_true", help='enables script verbosity')
  args = parser.parse_args()


  # -- Libraries to get, build, and install --
  libraries = []

  # -- Boost Library
  boostLibraryObj = BoostLibrary(args.boost)
  libraries.append( boostLibraryObj )  

  # -- OpenCL Headers
  openCLHeaderObj = OpenCLHeaders(args.opencl)
  libraries.append( openCLHeaderObj ) 

  # -- OpenCL ICD Library
  icdLibraryObj = ICDLibrary(args.icd)
  libraries.append( icdLibraryObj )       

# -- gtest Library
  gtestLibraryObj = GTestLibrary(args.gtest)
  libraries.append( gtestLibraryObj )

  # -- Evaluate the options --------------------------------------------------
  verbose = args.verbose

  # -- Validate --
  if args.validate_all_requirements == True:
    validateTools( True, verbose )
    validateLibraries( libraries, True, verbose )
    buildConflicts( libraries, True, True, verbose)
    return False

  # -- Check for existing tools ----------------------------------------------
  if validateTools( True, verbose ) == False:
    print ("")
    print ("ERROR: Missing supporting build tools.")
    return True

  # -- Check for existing build conflicts ------------------------------------
  if buildConflicts( libraries, False, True, verbose ) == True:
    print ("")
    print ("ERROR: Existing build present for the given library to be installed.")
    print ("       Please remove this directory prior re-installing the library.")
    return True

  # -- Check library dependencies --------------------------------------------
  if icdLibraryObj.skipBuild() == False and openCLHeaderObj.skipBuild() == True:
    if openCLHeaderObj.isInstalled(False, verbose) == False:
      print ("")
      print ("ERROR: The library '" + icdLibraryObj.getName() + "' is dependent on the library '" + openCLHeaderObj.getName() +"'")
      print ("       which isn't installed nor to be installed on the command line.")
      print ("")
      print ("       Please either install it or add it as an option to be installed.")
      return True

  # -- Install the given libraries -------------------------------------------
  if installLibraries( libraries, verbose) == True:
    return True

  return False

#==============================================================================
# -- Class: BoostLibrary ------------------------------------------------------
#==============================================================================
class BoostLibrary:
  # --
  def __init__(self, skip):
    self.install_dir = XRT_LIBRARY_INSTALL_DIR
    self.root_build_dir = XRT_LIBRARY_BUILD_DIR
    self.tag_version = "boost-1.70.0"
    self.library_version = "1_70"
    self.skipBuildInstall = skip
    self.buildDir = "boost"

  # --
  def getName(self):
    return self.tag_version

  # --
  def isInstalled(self, echo, verbose):
    # Override echo if verbosity is enabled
    if verbose == True:
      echo = True;

    boostInclude = os.path.join(self.install_dir, "include", "boost-" + self.library_version)

    if echo == True:
      print ("  " + self.getName() + " .................................... ", end='')

    if not os.path.exists(boostInclude) or not os.path.isdir(boostInclude):
      if echo == True:
        print ("[not installed]")
        if verbose == True:
          print ("    Expected directory: " + boostInclude)
      return False

    if echo == True:
      print ("[installed]")
      if verbose == True:
        print ("    Found directory: " + boostInclude)

    return True

  # --
  def isBuildPresent(self, echo, verbose):
    # Override echo if verbosity is enabled
    if verbose == True:
      echo = True;

    if echo == True:
      print ("  " + self.getName() + " build directory .................... ", end='')

    boostBuild = os.path.join(self.root_build_dir, "boost")
    if not os.path.exists(boostBuild) or not os.path.isdir(boostBuild):
      if echo == True:
        print ("[not found]")
        if verbose == True:
          print ("    Expected directory: " + boostBuild)
      return False

    if echo == True:
      print ("[found]")
      if verbose == True:
        print ("    Found directory: " + boostBuild)

    return True


  # --
  def getBuildAndInstallLibraryMinimum(self, verbose):
    # --
    print ("\n============================================================== ")
    print ("Starting Minimum Boost build")
    print ("Currently, this flow only does a 'minimum' library build as")
    print ("opposed to a 'minimum' git clone.")
    print ("============================================================== ")

    if verbose == True:
      print ("Creating Build Directory: " + self.root_build_dir)

    pathlib.Path(self.root_build_dir).mkdir(parents=True, exist_ok=True) 
    os.chdir(self.root_build_dir)

    if verbose == True:
      print ("Retrieving git repository...")

    cloneCmd = "git clone --recursive --branch=" + self.tag_version + "  https://github.com/boostorg/boost.git"
    print (cloneCmd)
    os.system(cloneCmd)

    boostCloneDir = os.path.join(self.root_build_dir, self.buildDir)
    if os.path.exists(boostCloneDir) == False:
      print ("ERROR: Unable to successfully retrieve the boost repository")
      print ("       Boost clone directory missing: " + boostCloneDir)
      return True

    # -- Build and release the boost repository
    if verbose == True:
      print ("Invoking boost bootstrap.bat...")

    os.chdir(boostCloneDir)
    cmd = "bootstrap.bat"
    print (cmd)
    os.system(cmd)

    if verbose == True:
      print ("Building and installing the boost library...")

    cmd = "b2 -j6 install toolset=msvc-14.1 --prefix=" + self.install_dir + " --build-type=complete address-model=64 architecture=x86 link=static threading=multi --with-filesystem --with-program_options --with-system"
    print (cmd)
    os.system(cmd)

    return False

  # --
  def getBuildAndInstallLibrary(self, verbose):
    
    if self.skipBuildInstall == "minimal":
      return self.getBuildAndInstallLibraryMinimum(verbose)

    # --
    print ("\n============================================================== ")
    print ("Starting FULL Boost build")
    print ("============================================================== ")

    if verbose == True:
      print ("Creating Build Directory: " + self.root_build_dir)

    pathlib.Path(self.root_build_dir).mkdir(parents=True, exist_ok=True) 
    os.chdir(self.root_build_dir)

    if verbose == True:
      print ("Retrieving git repository...")

    cloneCmd = "git clone --recursive --branch=" + self.tag_version + "  https://github.com/boostorg/boost.git"
    print (cloneCmd)
    os.system(cloneCmd)

    boostCloneDir = os.path.join(self.root_build_dir, self.buildDir)
    if os.path.exists(boostCloneDir) == False:
      print ("ERROR: Unable to successfully retrieve the boost repository")
      print ("       Boost clone directory missing: " + boostCloneDir)
      return True

    # -- Build and release the boost repository
    if verbose == True:
      print ("Invoking boost bootstrap.bat...")

    os.chdir(boostCloneDir)
    cmd = "bootstrap.bat"
    print (cmd)
    os.system(cmd)

    if verbose == True:
      print ("Building and installing the boost library...")

    cmd = "b2 -j6 install toolset=msvc-14.1 --prefix=" + self.install_dir + " --build-type=complete"
    print (cmd)
    os.system(cmd)

    return False

  # --
  def skipBuild(self):
    return self.skipBuildInstall == 'skip'


      


#==============================================================================
# -- Class: OpenCLLibrary -----------------------------------------------------
#==============================================================================
class OpenCLHeaders:
  # --
  def __init__(self, skip):
    self.install_dir = XRT_LIBRARY_INSTALL_DIR
    self.root_build_dir = XRT_LIBRARY_BUILD_DIR
    self.skipBuildInstall = skip
    self.headerDir = "CL"
    self.buildDir = "OpenCL-Headers"

  # --
  def getName(self):
    return "Khronos OpenCL Headers"

  # --
  def isInstalled(self, echo, verbose):
    # Override echo if verbosity is enabled
    if verbose == True:
      echo = True;

    openCLInclude = os.path.join(self.install_dir, "include", self.headerDir)

    if echo == True:
      print ("  Khronos OpenCL headers .......................... ", end='')

    headerFilePath1 = os.path.join(openCLInclude, "cl.h")
    if not os.path.exists(headerFilePath1) or not os.path.isfile(headerFilePath1):
      if echo == True:
        print ("[not installed]")
        if verbose == True:
          print ("    Missing header: " + headerFilePath1)
      return False

    headerFilePath2 = os.path.join(openCLInclude, "cl_ext.h")
    if not os.path.exists(headerFilePath2) or not os.path.isfile(headerFilePath2):
      if echo == True:
        print ("[not installed]")
        if verbose == True:
          print ("    Missing header: " + headerFilePath2)
      return False

    headerFilePath3 = os.path.join(openCLInclude, "cl2.hpp")
    if not os.path.exists(headerFilePath1) or not os.path.isfile(headerFilePath1):
      if echo == True:
        print ("[not installed]")
        if verbose == True:
          print ("    Missing header: " + headerFilePath3)
      return False

    if echo == True:
      print ("[installed]")
      if verbose == True:
        print ("    Found header: " + headerFilePath1)
        print ("    Found header: " + headerFilePath2)

    return True

  # --
  def isBuildPresent(self, echo, verbose):
    # Override echo if verbosity is enabled
    if verbose == True:
      echo = True;

    if echo == True:
      print ("  Khronos OpenCL headers build directory .......... ", end='')

    buildDir = os.path.join(self.root_build_dir, self.buildDir)
    if not os.path.exists(buildDir) or not os.path.isdir(buildDir):
      if echo == True:
        print ("[not found]")
        if verbose == True:
          print ("    Expected directory: " + buildDir)
      return False

    if echo == True:
      print ("[found]")
      if verbose == True:
        print ("    Found directory: " + buildDir)

    return True

  # --
  def getBuildAndInstallLibrary(self, verbose):
    # -- 
    print ("\n============================================================== ")
    print ("Starting OpenCL-Headers build")
    print ("============================================================== ")

    if verbose == True:
      print ("Creating Build Directory: " + self.root_build_dir)

    pathlib.Path(self.root_build_dir).mkdir(parents=True, exist_ok=True) 
    os.chdir(self.root_build_dir)

    if verbose == True:
      print ("Retrieving git repository...")

    gitDir = os.path.join(self.root_build_dir, self.buildDir)
    cloneCmd = "git clone https://github.com/KhronosGroup/OpenCL-Headers.git " + gitDir
    print (cloneCmd)
    os.system(cloneCmd)

    if os.path.exists(gitDir) == False:
      print ("ERROR: Unable to successfully retrieve the Khronos OpenCl-Headers repository")
      print ("       Khronos OpenCL-Headers clone directory missing: " + gitDir)
      return True

    # -- Get the cl2.hpp file
    destcl2hpp = os.path.join(self.root_build_dir, self.buildDir, "CL", "cl2.hpp")
    urllib.request.urlretrieve("https://github.com/KhronosGroup/OpenCL-CLHPP/releases/download/v2.0.10/cl2.hpp", destcl2hpp)

    # -- Copy the header files
    srcDir = os.path.join(gitDir, "CL")
    destDir = os.path.join(self.install_dir, "include", "CL")

    if verbose == True:
      print ("Creating destination directory: " + destDir)

    pathlib.Path(destDir).mkdir(parents=True, exist_ok=True) 

    if verbose == True:
      print ("Copying header directory.")
      print ("   Source      : " + srcDir)
      print ("   Destination : " + destDir)

    distutils.dir_util.copy_tree(srcDir, destDir)

    return False

  # --
  def skipBuild(self):
    return self.skipBuildInstall == False



#==============================================================================
# -- Class: ICDLibrary --------------------------------------------------------
#==============================================================================
class ICDLibrary:
  # --
  def __init__(self, skip):
    self.install_dir = XRT_LIBRARY_INSTALL_DIR
    self.root_build_dir = XRT_LIBRARY_BUILD_DIR
    self.skipBuildInstall = skip
    self.headerDir = "CL"
    self.buildDir = "OpenCL-ICD-Loader"

  # --
  def getName(self):
    return "Khronos OpenCL ICD"

  # --
  def isInstalled(self, echo, verbose):
    # Override echo if verbosity is enabled
    if verbose == True:
      echo = True;

    if echo == True:
      print ("  Khronos OpenCL ICD Libraries .................... ", end='')

    libraryFilePath1 = os.path.join(self.install_dir, "bin", "OpenCL.dll")
    if not os.path.exists(libraryFilePath1) or not os.path.isfile(libraryFilePath1):
      if echo == True:
        print ("[not installed]")
        if verbose == True:
          print ("    Missing library: " + libraryFilePath1)
      return False

    libraryFilePath2 = os.path.join(self.install_dir, "lib", "OpenCL.lib")
    if not os.path.exists(libraryFilePath2) or not os.path.isfile(libraryFilePath2):
      if echo == True:
        print ("[not installed]")
        if verbose == True:
          print ("    Missing library: " + libraryFilePath2)
      return False

    if echo == True:
      print ("[installed]")
      if verbose == True:
        print ("    Library found: " + libraryFilePath1)
        print ("    Library found: " + libraryFilePath2)

    return True

  # --
  def isBuildPresent(self, echo, verbose):
    # Override echo if verbosity is enabled
    if verbose == True:
      echo = True;

    if echo == True:
      print ("  Khronos OpenCL ICD build directory .............. ", end='')

    buildDir = os.path.join(self.root_build_dir, self.buildDir)
    if not os.path.exists(buildDir) or not os.path.isdir(buildDir):
      if echo == True:
        print ("[not found]")
        if verbose == True:
          print ("    Expected directory: " + buildDir)
      return False

    if echo == True:
      print ("[found]")
      if verbose == True:
        print ("    Found directory: " + buildDir)

    return True


  # --
  def getBuildAndInstallLibrary(self, verbose):
    # -- 
    print ("\n============================================================== ")
    print ("Starting OpenCL-ICD-Loader build")
    print ("============================================================== ")

    if verbose == True:
      print ("Creating Build Directory: " + self.root_build_dir)

    pathlib.Path(self.root_build_dir).mkdir(parents=True, exist_ok=True) 
    os.chdir(self.root_build_dir)

    if verbose == True:
      print ("Retrieving git repository...")

    gitDir = os.path.join(self.root_build_dir, self.buildDir)
    cloneCmd = "git clone https://github.com/KhronosGroup/OpenCL-ICD-Loader.git " + gitDir
    print (cloneCmd)
    os.system(cloneCmd)

    if os.path.exists(gitDir) == False:
      print ("ERROR: Unable to successfully retrieve the Khronos OpenCl ICD Loader repository")
      print ("       Khronos OpenCL ICD Loader clone directory missing: " + gitDir)
      return True

    # -- Copy the header files
    srcDir = os.path.join(self.install_dir, "include", self.headerDir)
    dstDir = os.path.join(gitDir, "inc", self.headerDir)

    if verbose == True:
      print ("Creating destination directory: " + dstDir)

    pathlib.Path(dstDir).mkdir(parents=True, exist_ok=True) 

    if verbose == True:
      print ("Copying header directory.")
      print ("   Source      : " + srcDir)
      print ("   Destination : " + dstDir)

    distutils.dir_util.copy_tree(srcDir, dstDir)

    # -- Build the library
    buildingDir = pathlib.Path(gitDir, "build")

    if verbose == True:
      print ("Creating build directory: " + str(buildingDir))

    os.mkdir(buildingDir)
    os.chdir(buildingDir)

    # -- Create the build scripts
    if verbose == True:
      print ("Invoking cmake to create the build scripts...")

    cmd = "cmake -G \"Visual Studio 15 2017 Win64\" -DOPENCL_ICD_LOADER_REQUIRE_WDK=0 -DCMAKE_INSTALL_PREFIX=" + self.install_dir + " .."
    print (cmd)
    os.system(cmd)

    # -- Create the release library
    if verbose == True:
      print ("Invoking cmake build the release library...")

    cmd = "cmake --build . --verbose --config Release"
    print (cmd)
    os.system(cmd)

    # -- Install the library
    if verbose == True:
      print ("Invoking cmake build to install the library...")

    cmd = "cmake --build . --verbose --config Release --target install"
    print (cmd)
    os.system(cmd)

    return False;

  # --
  def skipBuild(self):
    return self.skipBuildInstall == False

#==============================================================================
# -- Class: GTestLibrary --------------------------------------------------------
#==============================================================================
class GTestLibrary:
  # --
  def __init__(self, skip):
    self.install_dir = XRT_LIBRARY_INSTALL_DIR
    self.root_build_dir = XRT_LIBRARY_BUILD_DIR
    self.skipBuildInstall = skip
    self.headerDir = "gtest"
    self.buildDir = "googletest"

  # --
  def getName(self):
    return "GTestLibrary"

  # --
  def isInstalled(self, echo, verbose):
    # Override echo if verbosity is enabled
    if verbose == True:
      echo = True;

    gtestInclude = os.path.join(self.install_dir, "include", "gtest")

    if echo == True:
      print ("  " + self.getName() + " .................................... ", end='')

    if not os.path.exists(gtestInclude) or not os.path.isdir(gtestInclude):
      if echo == True:
        print ("[not installed]")
        if verbose == True:
          print ("    Expected directory: " + gtestInclude)
      return False

    if echo == True:
      print ("[installed]")
      if verbose == True:
        print ("    Found directory: " + gtestInclude)

    return True

  # --
  def isBuildPresent(self, echo, verbose):
    # Override echo if verbosity is enabled
    if verbose == True:
      echo = True;

    if echo == True:
      print ("  " + self.getName() + " build directory .................... ", end='')

    gtestBuild = os.path.join(self.root_build_dir, "googletest")
    if not os.path.exists(gtestBuild) or not os.path.isdir(gtestBuild):
      if echo == True:
        print ("[not found]")
        if verbose == True:
          print ("    Expected directory: " + gtestBuild)
      return False

    if echo == True:
      print ("[found]")
      if verbose == True:
        print ("    Found directory: " + gtestBuild)

    return True


  # --
  def getBuildAndInstallLibrary(self, verbose):
    # -- 
    print ("\n============================================================== ")
    print ("Starting GTest Library build")
    print ("============================================================== ")

    if verbose == True:
      print ("Creating Build Directory: " + self.root_build_dir)

    pathlib.Path(self.root_build_dir).mkdir(parents=True, exist_ok=True) 
    os.chdir(self.root_build_dir)

    if verbose == True:
      print ("Retrieving git repository...")

    gitDir = os.path.join(self.root_build_dir, self.buildDir)
    cloneCmd = "git clone https://github.com/google/googletest.git "+ gitDir
    print (cloneCmd)
    os.system(cloneCmd)

    if os.path.exists(gitDir) == False:
      print ("ERROR: Unable to successfully retrieve the Google-Test repository")
      print ("       Google-Test clone directory missing: " + gitDir)
      return True

    # -- Copy the header files
    srcDir = os.path.join(gitDir, "googletest", "include", "gtest")
    dstDir = os.path.join(XRT_LIBRARY_INSTALL_DIR, "include", "gtest")

    if verbose == True:
      print ("Creating destination directory: " + dstDir)

    pathlib.Path(dstDir).mkdir(parents=True, exist_ok=True) 

    if verbose == True:
      print ("Copying header directory.")
      print ("   Source      : " + srcDir)
      print ("   Destination : " + dstDir)

    distutils.dir_util.copy_tree(srcDir, dstDir)

    # -- Build the library
    buildingDir = pathlib.Path(gitDir, "build")

    if verbose == True:
      print ("Creating build directory: " + str(buildingDir))

    os.mkdir(buildingDir)
    os.chdir(buildingDir)

    # -- Create the build scripts
    if verbose == True:
      print ("Invoking cmake to create the build scripts...")

    cmd = "cmake -G \"Visual Studio 15 2017 Win64\" .."
    print (cmd)
    os.system(cmd)

	# -- Copying cmake files (GTestConfig, GTestConfigVersion)
    srcDir = os.path.join(buildingDir, "googletest", "generated")
    dstDir = os.path.join(XRT_LIBRARY_INSTALL_DIR, "lib", "cmake", "gtest")
    pathlib.Path(dstDir).mkdir(parents=True, exist_ok=True)
    distutils.dir_util.copy_tree(srcDir, dstDir)

    # -- Create the release library
    if verbose == True:
      print ("Invoking cmake build the release library...")

    cmd = "cmake --build . --verbose --config Release"
    print (cmd)
    os.system(cmd)

	# -- Create the Debug library
    if verbose == True:
      print ("Invoking cmake build the Debug library...")

    cmd = "cmake --build . --verbose --config Debug"
    print (cmd)
    os.system(cmd)

    # -- Copying Release .lib files
    if verbose == True:
      print ("Copying Release libraries...")

    srcDir = os.path.join(buildingDir, "lib", "Release")
    dstDir = os.path.join(XRT_LIBRARY_INSTALL_DIR, "lib")
    distutils.dir_util.copy_tree(srcDir, dstDir)

	# -- Copying Debug .lib files
    if verbose == True:
      print ("Copying Debug libraries...")

    srcDir = os.path.join(buildingDir, "lib", "Debug")
    dstDir = os.path.join(XRT_LIBRARY_INSTALL_DIR, "lib")
    distutils.dir_util.copy_tree(srcDir, dstDir)

    return False;

  # --
  def skipBuild(self):
    return self.skipBuildInstall == False

# ============================================================================
# Helper methods
# ============================================================================

# -- which --------------------------------------------------------------------
# Look for the executable in the path
def which( program ):
    # -- isExe ----------------------------------------------------------------
    def isExe(filePath):
        return os.path.isfile(filePath) and os.access(filePath, os.X_OK)

    filePath, fileName = os.path.split(program)
    if filePath:
        if isExe( program ):
            return program
    else:
        for aPath in os.environ["PATH"].split(os.pathsep):
            exeFile = os.path.join(aPath, program)
            if isExe(exeFile):
                return exeFile

    return None

# -- validateTools -------------------------------------------------------------
def validateTools( echo, verbose ):
  # -- supporting method
  def toolFoundNotFound(program, echo, verbose):
    exeFile = which(program)
    if exeFile is None:
      print ("[not found]")
      return False
    else:
      print ("[found]")
      if verbose == True:
        print ("    Path: " + exeFile)
      return True


  allToolsInstalled = True            # Assume all of the tools are installed
  if echo == True:
    print ("\n---------------------------------------------------------------------------")
    print ("Validating the installation XRT Windows supporting applications.")
    print ("---------------------------------------------------------------------------")

  # -- CL 
  print ("  CL .............................................. ", end='')
  if toolFoundNotFound("cl.exe", echo, verbose) == False:
    allToolsInstalled = False;

  # -- git
  print ("  git ............................................. ", end='')
  if toolFoundNotFound("git.exe", echo, verbose) == False:
    allToolsInstalled = False;

  # -- CMAKE
  print ("  CMAKE ........................................... ", end='')
  if toolFoundNotFound("cmake.exe", echo, verbose) == False:
    allToolsInstalled = False;

  return allToolsInstalled


# -- validateLibraries --------------------------------------------------------
def validateLibraries( libraries, echo, verbose ):
  if echo == True:
    print ("\n---------------------------------------------------------------------------")
    print ("Examining and validating the installation XRT Windows supporting libraries.")
    print ("---------------------------------------------------------------------------")

  for library in libraries:
    library.isInstalled( echo, verbose )


# -- buildConflicts -----------------------------------------------------------
def buildConflicts( libraries, checkAll, echo, verbose ):
  print ("\n---------------------------------------------------------------------------")
  print ("Examining system for existing library build directories.")
  print ("---------------------------------------------------------------------------")

  buildConflicts = False;
  # Examine only the libraries that are about to be installed
  for library in libraries:
    if checkAll == False and library.skipBuild() == True:
      continue

    if library.isBuildPresent(echo, verbose):
      buildConflicts = True

  return buildConflicts

# -- installLibraries ---------------------------------------------------------
def installLibraries( libraries, verbose ):
  for library in libraries:
    if library.skipBuild() == True:
      continue

    print ("\n---------------------------------------------------------")
    print ("Installing Library: " + library.getName())
    print ("---------------------------------------------------------")
    if library.getBuildAndInstallLibrary( verbose ) == True:
       print ("ERROR: Installing " + library.getName())
       return True

  return False


# -- Start executing the script functions
if __name__ == '__main__':
  if main() == True:
    print ("\nError(s) occurred.")
    exit(1)





##
 # Copyright (C) 2018-2020 Xilinx, Inc
 # Helper routines for Python based XRT tests
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You may
 # not use this file except in compliance with the License. A copy of the
 # License is located at
 #
 #     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
##
import sys
import getopt
import struct
import ctypes
import uuid
# XRT modules imported from PYTHONPATH
from xclbin_binding import *
from xrt_binding import *
from ert_binding import *


class Options(object):
    def __init__(self):
        self.DATA_SIZE = 1024
        self.sharedLibrary = None
        self.bitstreamFile = None
        self.halLogFile = None
        self.alignment = 4096
        self.option_index = 0
        self.index = 0
        self.cu_index = 0
        self.verbose = False
        self.handle = None
        self.xcl_handle = None
        self.first_mem = -1
        self.cu_base_addr = -1
        self.xuuid = uuid.uuid4()
        self.kernels = []

    def getOptions(self, argv):
        try:
            opts, args = getopt.getopt(argv[1:], "k:l:a:c:d:vhe", ["bitstream=", "hal_logfile=", "alignment=",
                                                                   "cu_index=", "device=", "verbose", "help", "ert"])
        except getopt.GetoptError:
            print(self.printHelp())
            sys.exit(2)

        for o, arg in opts:
            if o in ("--bitstream", "-k"):
                self.bitstreamFile = arg
            elif o in ("--hal_logfile", "-l"):
                self.halLogFile = arg
            elif o in ("--alignment", "-a"):
                print("-a/--alignment switch is not supported")
            elif o in ("--cu_index", "-c"):
                self.cu_index = int(arg)
            elif o in ("--device", "-d"):
                self.index = int(arg)
            elif o in ("--help", "-h"):
                print(self.printHelp())
            elif o == "-v":
                self.verbose = True
            elif o in ("-e", "--ert"):
                print("-e/--ert switch is not supported")
            else:
                assert False, "unhandled option"

        if self.bitstreamFile is None:
            raise RuntimeError("No bitstream specified")

        if self.halLogFile:
            print("Log files are not supported on command line, Please use xrt.ini to specify logging configuration")
        print("Host buffer alignment " + str(self.alignment) + " bytes")
        print("Compiled kernel = " + self.bitstreamFile)

    def printHelp(self):
        print("usage: %s [options] -k <bitstream>")
        print("  -k <bitstream>")
        print("  -d <device_index>")
        print("  -c <cu_index>")
        print("  -v")
        print("  -h")
        print("")
        print("* Bitstream is required")

def initXRT(opt):
    deviceInfo = xclDeviceInfo2()
    if opt.index >= xclProbe():
        raise RuntimeError("Incorrect device index")

    opt.handle = xrtDeviceOpen(opt.index)
    opt.xcl_handle = xrtDeviceToXclDevice(opt.handle)

    xclGetDeviceInfo2(opt.xcl_handle, ctypes.byref(deviceInfo))

    if sys.version_info[0] == 3:
        print("Shell = %s" % deviceInfo.mName)
        print("Index = %d" % opt.index)
        print("PCIe = GEN%d x %d" % (deviceInfo.mPCIeLinkSpeed, deviceInfo.mPCIeLinkWidth))
        print("OCL Frequency = (%d, %d) MHz" % (deviceInfo.mOCLFrequency[0], deviceInfo.mOCLFrequency[1]))
        print("DDR Bank = %d" % deviceInfo.mDDRBankCount)
        print("Device Temp = %d C" % deviceInfo.mOnChipTemp)
        print("MIG Calibration = %s" % deviceInfo.mMigCalib)
    else:
        print("Shell = %s") % deviceInfo.mName
        print("Index = %s") % opt.index
        print("PCIe = GEN%s" + " x %s") % (deviceInfo.mPCIeLinkSpeed, deviceInfo.mPCIeLinkWidth)
        print("OCL Frequency = %s MHz") % deviceInfo.mOCLFrequency[0]
        print("DDR Bank = %d") % deviceInfo.mDDRBankCount
        print("Device Temp = %d C") % deviceInfo.mOnChipTemp
        print("MIG Calibration = %s") % deviceInfo.mMigCalib

    tempFileName = opt.bitstreamFile

    with open(tempFileName, "rb") as f:
        data = bytearray(os.path.getsize(tempFileName))
        f.readinto(data)
        f.close()
        blob = (ctypes.c_char * len(data)).from_buffer(data)
        xbinary = axlf.from_buffer(data)
        if xbinary.m_magic.decode("utf-8") != "xclbin2":
            raise RuntimeError("Invalid Bitsream")

        xclLoadXclBin(opt.xcl_handle, blob)
        print("Finished downloading bitstream %s" % opt.bitstreamFile)

        myuuid = memoryview(xbinary.m_header.u2.uuid)[:]
        opt.xuuid = uuid.UUID(bytes=myuuid.tobytes())
        head = wrap_get_axlf_section(blob, AXLF_SECTION_KIND.IP_LAYOUT)
        layout = ip_layout.from_buffer(data, head.contents.m_sectionOffset)

        if opt.cu_index > layout.m_count:
            raise RuntimeError("Can't determine cu base address")

        ip = (ip_data * layout.m_count).from_buffer(data, head.contents.m_sectionOffset + 8)

        for i in range(layout.m_count):
            if (ip[i].m_type != 1):
                continue
            opt.cu_base_addr = ip[i].ip_u1.m_base_address
            opt.kernels.append(ctypes.cast(ip[i].m_name, ctypes.c_char_p).value)
            print("CU[%d] %s @0x%x" % (i, opt.kernels[-1], opt.cu_base_addr))

        head = wrap_get_axlf_section(blob, AXLF_SECTION_KIND.MEM_TOPOLOGY)
        topo = mem_topology.from_buffer(data, head.contents.m_sectionOffset)
        mem = (mem_data * topo.m_count).from_buffer(data, head.contents.m_sectionOffset + 8)

        for i in range(topo.m_count):
            print("[%d] %s @0x%x" % (i, ctypes.cast(mem[i].m_tag, ctypes.c_char_p).value, mem[i].mem_u2.m_base_address))
            if (mem[i].m_used == 0):
                continue
            opt.first_mem = i
            break

    return 0



"""

 Copyright (C) 2016-2020 Xilinx, Inc

 Licensed under the Apache License, Version 2.0 (the "License"). You may
 not use this file except in compliance with the License. A copy of the
 License is located at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
  License for the specific language governing permissions and limitations
 under the License.

"""

import os
import sys
import platform
import subprocess

from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext


class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=''):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def build_extension(self, ext):
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))

        cmake_args = ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + extdir,
                      '-DPYTHON_EXECUTABLE=' + sys.executable]
        cmake_args += ['-DCMAKE_BUILD_TYPE=' + 'Release']
        
        build_args = ['--config', 'Release']

        env = os.environ.copy()
        env['CXXFLAGS'] = '{} -DVERSION_INFO=\\"{}\\"'.format(env.get('CXXFLAGS', ''),
                                                               self.distribution.get_version())
        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
            
        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args, cwd=self.build_temp, env=env)
        subprocess.check_call(['cmake', '--build', '.'] + build_args, cwd=self.build_temp)


setup(
    name='pyxrt',
    version='1.0.0',
    author='Graham Schelle',
    author_email='graham.schelle@xilinx.com',
    description="Pybind11 package for Xilinx's XRT",
    ext_modules=[CMakeExtension('pyxrt')],
    cmdclass=dict(build_ext=CMakeBuild),
    zip_safe=False,
)



import glob
import yaml
import os, os.path
from subprocess import call
from subprocess import Popen,PIPE,STDOUT

HALLIB = "libxclgemdrv.so"
DSA_PLATFORM = "xilinx_xil-accel-rd-ku115_4ddr-xpr_4_0"
date = '0503'
output = '/scratch/umangp/sprite_' + date
dsa = 'ku115'
ver = '4_0'
#base = '/proj/rdi-xco/fisusr/no_backup/results/sdx_2017.1/sdx_ll_hw/'
base = '/proj/rdi-xco/fisusr/no_backup/results/sdx_2017.1/sdx_unit_hw/'


def get_args(ymlfile):
	#print get_args.__name__
	#print ymlfile
	yml = glob.glob(ymlfile)
	#print yml
	f = open(yml[0])
	data = yaml.safe_load(f)
	f.close()
	return data['args']
	

def main():
	file = open('py.log', 'w+')
	os.environ["HALLIB"]= HALLIB
	os.environ["DSA_PLATFORM"]= DSA_PLATFORM
	call(["mkdir", "-p", output])
	search = base + '*' + dsa + '*' + ver
	result = glob.glob(search)
	
	try:

		for i in result:
			dirname = os.path.split(i)[1]
			#print dirname
			
			design = dirname[:10]
			#print design
			
			
			abcd= raw_input()
			if abcd == "n"
				continue
			
			new_dir = output + os.sep + design
			print new_dir
			
			call(["mkdir", "-p", new_dir])		
			
			ymlfile = i + os.sep + '*' + date + '*' + os.sep + 'sdainfo.yml'
			args = get_args(ymlfile)
			print args
			
			yml = glob.glob(ymlfile)
			
			call(["cp", "-f", yml[0], new_dir])
			
			resdir = os.path.split(yml[0])[0]
			
			xbinst = resdir + os.sep + 'xbinst' + os.sep + '*'
			#print xbinst
			
			xbinst_files = glob.glob(xbinst)
			#for filename in os.listdir(xbinst):
			for filename in xbinst_files:
				#print "UKP: " + filename
				if ".log" in filename:
					continue
				if os.path.isfile(filename):
					call(["cp", "-f", filename, new_dir])
					#print "UKP2: " +filename
			
			
			print "......Running the test now... "
			#cmd = "./host.exe " + args +  " | tee output.log"
			#print cmd
			
			my_env = os.environ.copy()
			my_env["HALLIB"]= HALLIB
			my_env["DSA_PLATFORM"]= DSA_PLATFORM
			
			p = Popen(["./host.exe " + args ], shell=True, cwd=new_dir, env=my_env)
			#p = Popen(["./host.exe " + args + " | tee output.log" + "; dmesg -c| tee dmesg.log"], shell=True, cwd=new_dir, env=my_env)
			p.wait()
			#sts = Popen(["./host.exe ", args, " | tee output.log" ], shell=False, cwd=new_dir, env=my_env)
			abcd= raw_input()
			
			file.write("\nReturn code of " + design + " : " + str(p.returncode))
			print "Return code of " + design + " : " + str(p.returncode)
			print "========================================================"
	except:
		print "catching exception..."
		file.close()

	
if __name__ == "__main__":
	main()
	



import gdb
import sys
import os

script_path = os.path.abspath(os.path.dirname(__file__))
sys.path.append(script_path)

from appdebug import infCallUtil

class printSPMInfo (infCallUtil):
	"Print the Performance Monitor counters"
	def invoke (self, arg, jsonformat):
		fargs = []
		free_args,spm_ptr,errmsg = self.callfunc_verify("appdebug::clGetDebugCounters",fargs, "AIM")

		if (spm_ptr == 0):
			if (jsonformat == True):
				print ("[{{\"info\": \"{}\"}}]".format (errmsg))
			else :
				print ("Error reading AIMs : " + (errmsg))
			return 

		if (jsonformat):
			stdstr = self.callmethod(spm_ptr,"getstring",[1, 1]);
			strout = stdstr['_M_dataplus']['_M_p'].string();
			print (strout)
		else:
			stdstr = self.callmethod(spm_ptr,"getstring",[1, 0]);
			strout = stdstr['_M_dataplus']['_M_p'].string();
			print (strout)

		# free the allocated vector
		self.callfunc("appdebug::clFreeAppDebugView",free_args)

obj_spm = printSPMInfo ()

class printSSPMInfo (infCallUtil):
	"Print the Streaming Performance Monitor counters"
	def invoke (self, arg, jsonformat):
		fargs = []
		free_args,sspm_ptr,errmsg = self.callfunc_verify("appdebug::clGetDebugStreamCounters",fargs, "ASM")

		if (sspm_ptr == 0):
			if (jsonformat == True):
				print ("[{{\"info\": \"{}\"}}]".format (errmsg))
			else :
				print ("Error reading ASMs: " + (errmsg))
			return				

		if (jsonformat):
			stdstr = self.callmethod(sspm_ptr,"getstring",[1, 1]);
			strout = stdstr['_M_dataplus']['_M_p'].string();
			print (strout)
		else:
			stdstr = self.callmethod(sspm_ptr,"getstring",[1, 0]);
			strout = stdstr['_M_dataplus']['_M_p'].string();
			print (strout)

		# free the allocated vector
		self.callfunc("appdebug::clFreeAppDebugView",free_args)
obj_sspm = printSSPMInfo()

class printSAMInfo (infCallUtil):
	"Print the Accelerator Monitor counters"
	def invoke (self, arg, jsonformat):
		fargs = []
		free_args,sam_ptr,errmsg = self.callfunc_verify("appdebug::clGetDebugAccelMonitorCounters",fargs, "AM")

		if (sam_ptr == 0):
			if (jsonformat == True):
				print ("[{{\"info\": \"{}\"}}]".format (errmsg))
			else :
				print ("Error reading AMs : " + (errmsg))
			return				

		if (jsonformat):
			stdstr = self.callmethod(sam_ptr,"getstring",[1, 1]);
			strout = stdstr['_M_dataplus']['_M_p'].string();
			print (strout)
		else:
			stdstr = self.callmethod(sam_ptr,"getstring",[1, 0]);
			strout = stdstr['_M_dataplus']['_M_p'].string();
			print (strout)

		# free the allocated vector
		self.callfunc("appdebug::clFreeAppDebugView",free_args)
obj_sam = printSAMInfo()

class printLAPCInfo (infCallUtil):
	"Print the status of Lightweight AXI Protocol Checker"
	def invoke (self, arg, jsonformat):
		fargs = []
		free_args,lapc_ptr,errmsg = self.callfunc_verify("appdebug::clGetDebugCheckers",fargs, "LAPC")

		if (lapc_ptr == 0):
			if (jsonformat == True):
				print ("[{{\"info\": \"{}\"}}]".format (errmsg))
			else :
				print ("Error reading LAPCs: " + (errmsg))
			return				

		if (jsonformat):
			stdstr = self.callmethod(lapc_ptr,"getstring",[0, 1]);
			strout = stdstr['_M_dataplus']['_M_p'].string();
			print (strout)
		else :
			stdstr = self.callmethod(lapc_ptr,"getstring",[0, 0]);
			strout = stdstr['_M_dataplus']['_M_p'].string();
			print (strout)

		# free the allocated vector
		self.callfunc("appdebug::clFreeAppDebugView",free_args)

obj_lapc = printLAPCInfo ()

#########################################xstatus command#####################################
class xstatusPrefix(gdb.Command):
	"Xilinx command to show the status of all IPs"
	def __init__(self):
		super(xstatusPrefix, self).__init__(
						"xstatus",
						gdb.COMMAND_USER,
						gdb.COMPLETE_COMMAND,
						True)
xstatusPrefix()
class xstatusSPMInfo (gdb.Command,infCallUtil):
	"Print the Performance Monitor counters when available"
	def __init__ (self):
		super (xstatusSPMInfo, self).__init__ ("xstatus aim", 
                         gdb.COMMAND_USER)
	def invoke (self, arg, from_tty):
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print (e.message)
			return
		obj_spm.invoke(arg, 0)
xstatusSPMInfo()
class xstatusSSPMInfo (gdb.Command,infCallUtil):
	"Print the Streaming Performance Monitor counters when available"
	def __init__ (self):
		super (xstatusSSPMInfo, self).__init__ ("xstatus asm", 
                         gdb.COMMAND_USER)
	def invoke (self, arg, from_tty):
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print (e.message)
			return
		obj_sspm.invoke(arg, 0)
xstatusSSPMInfo()

class xstatusSAMInfo (gdb.Command,infCallUtil):
	"Print the Accelerator Monitor counters when available"
	def __init__ (self):
		super (xstatusSAMInfo, self).__init__ ("xstatus am", 
                         gdb.COMMAND_USER)
	def invoke (self, arg, from_tty):
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print (e.message)
			return
		obj_sam.invoke(arg, 0)
xstatusSAMInfo()

class xstatusLAPCInfo (gdb.Command,infCallUtil):
	"Print the status of Lightweight AXI Protocol Checkers when available"
	def __init__ (self):
		super (xstatusLAPCInfo, self).__init__ ("xstatus lapc", 
                         gdb.COMMAND_USER)
	def invoke (self, arg, from_tty):
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print (e.message)
			return
		obj_lapc.invoke(arg, 0)
xstatusLAPCInfo()

class xstatusAllInfo (gdb.Command,infCallUtil):
	"Print the status of all available IPs"
	def __init__ (self):
		super (xstatusAllInfo, self).__init__ ("xstatus all", 
                         gdb.COMMAND_USER)
	def invoke (self, arg, from_tty):
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print (e.message)
			return
		obj_spm.invoke(arg, 0)
		obj_sspm.invoke(arg, 0)
		obj_sam.invoke(arg, 0)
		obj_lapc.invoke(arg, 0)
xstatusAllInfo()

class xstatusJSONPrefix(gdb.Command):
	"Xilinx command to show the status of all IPs in JSON format"
	def __init__(self):
		super(xstatusJSONPrefix, self).__init__(
						"xstatus_json",
						gdb.COMMAND_USER,
						gdb.COMPLETE_COMMAND,
						True)
xstatusJSONPrefix()

class xstatusJSONAllInfo (gdb.Command,infCallUtil):
	"Print the status of all available IPs"
	def __init__ (self):
		super (xstatusJSONAllInfo, self).__init__ ("xstatus_json all", 
                         gdb.COMMAND_USER)
	def invoke (self, arg, from_tty):
		print ("{ \"aim\" : ")
		obj_spm.invoke(arg,1)
		print (",")
		print ("\"lapc\" : ")
		obj_lapc.invoke(arg,1)
		print ("}")
all_json_xstatus=xstatusJSONAllInfo()




import gdb
import sys
import os
script_path = os.path.abspath(os.path.dirname(__file__))
sys.path.append(script_path)

class infCallUtil():
	def printstdstring (self, val):
		print ((val['_M_dataplus']['_M_p']).string())
	
	def callmethod(self, val, fname, args):
		"make method in fname on the obj ref 'val'"
		methodcall = "( (" + str(val.type) + ")" + "(" + str(val).split()[0] + ") )" + "->" + fname + "("

		cnt = 0
		for arg in args:
			if (cnt):
				methodcall += ","
			try:
				methodcall += "( (" + str(arg.type) + ")" + "(" + str(arg).split()[0] + ") )"
			except AttributeError:
				methodcall += "(" + str(arg).split()[0] + ")"
			cnt+=1
		methodcall += ")"
		#print ("methodcall = ", methodcall)

		return gdb.parse_and_eval(methodcall)

	def callfunc(self, fname, args):
		"make call fname(args)"
		funccall = fname + "("
		cnt = 0
		for val in args:
			if (cnt):
				funccall += ","
			funccall += "( (" + str(val.type) + ")" + "(" + str(val).split()[0] + ") )"
			cnt+=1
		funccall += ")"
		#print ("funccall = ", funccall)
		return gdb.parse_and_eval(funccall)

	def callfunc_verify(self, fname, args, objname):
		adv_ptr = self.callfunc(fname, args)
	
		free_args = []
		free_args.append(adv_ptr)
   		#adv_ptr is app_debug_view <some_data>*
		if (adv_ptr == 0):
			#print ("Unexpected error while getting list of {}".format(objname))
			errmsg = "Unexpected error while getting list of {}".format(objname)
			return free_args, 0, errmsg

		isInvalid = self.callmethod(adv_ptr,"isInValid", [])
		if (isInvalid):
			errmsg = self.callmethod(adv_ptr,"geterrmsg", [])
			#self.printstdstring(errmsg)
			errmsg = ((errmsg['_M_dataplus']['_M_p']).string())
			# free the allocated debug view data
			self.callfunc("appdebug::clFreeAppDebugView",free_args)
			return free_args, 0, errmsg

		ptr = self.callmethod(adv_ptr,"getdata", [])
		if (ptr == 0):
			#print ("Unexpected error while getting {} info".format(objname))
			errmsg = "Unexpected error while getting {} info".format(objname)
			# free the allocated debug view data
			self.callfunc("appdebug::clFreeAppDebugView",free_args)
			return free_args, 0, errmsg
		return free_args, ptr, ""
	
	def check_app_debug_enabled(self):
		try:
			isEnabled = self.callfunc("appdebug::isAppdebugEnabled", [])
		except:
			raise ValueError("Application debug not available. Application debug will be loaded upon the first call to an OpenCL API (not a declaration of an OpenCL variable).")
		if str(isEnabled) == "false":
			raise ValueError("Application debug not enabled. Set attribute 'app_debug=true' under 'Debug' section of xrt.ini and restart application")
		return

class printEventDebugViewVector(infCallUtil):
	def __init__(self, typename, val):
		self.val = val
		self.typename = typename
		self.start = self.val['_M_impl']['_M_start']
		self.finish = self.val['_M_impl']['_M_finish']
		self.item = self.start
		self.str_out = ""

	def printstring(self):
		while (self.item != self.finish) :
			elem = self.item.dereference()
			margs = []
			self.callmethod(elem, "print", margs)
			self.item = self.item + 1

	def getstring(self, verbose, json):
		if self.item == self.finish :
			if (json):
				return "\"events\" : []"
			else:
				return "None"

		if (json) :
			self.str_out = self.str_out + "\"events\":["

		while (self.item != self.finish) :
			elem = self.item.dereference()
			margs = [verbose, json]
			s = self.callmethod(elem, "getstring", margs)
			#print "s  = ", s['_M_dataplus']['_M_p'] #std::string
			#print "s.string()  = ", (s['_M_dataplus']['_M_p']).string() #std::string
			self.str_out =  self.str_out + "{" + (s['_M_dataplus']['_M_p']).string() + "}"
			self.item = self.item + 1
			#don't append newline at the end of string, for better formating
			if (self.item != self.finish): 
				if (json):
					self.str_out += ","   #events array elements
				else:
					self.str_out += "\n"

		if (json):
			self.str_out += "]"; #events
		return self.str_out

#########################################xpq commands #####################################
class printOccupancy (infCallUtil):
	"Print the number of events in command queue"
	def invoke (self, arg, from_tty):
		fargs = []
		fargs.append(gdb.parse_and_eval("(cl_command_queue)"+arg))

		#print "fargs = ", fargs

		free_args,pair_ptr,errmsg = self.callfunc_verify("appdebug::clPrintCmdQOccupancy",fargs, "cl_command_queue")
		#print ("free_args = {} pair_ptr = {} errmsg = {}".format(free_args,pair_ptr,errmsg))
		if (pair_ptr == 0):
			print (errmsg)
			return

		val = pair_ptr.dereference()
		#print (ret1)
		print ("Queued: {} Submitted: {}".format(val['first'], val['second']))

		# free the allocated vector
		self.callfunc("appdebug::clFreeAppDebugView",free_args)

obj_po = printOccupancy ()

class printSubmitted (infCallUtil):
	"Print the submitted commands"

	def invoke (self, arg, jsonformat):
		fargs = []
		fargs.append(gdb.parse_and_eval("(cl_command_queue)"+arg))

		free_args, vec_ptr, errmsg = self.callfunc_verify("appdebug::clPrintCmdQSubmitted",fargs, "submitted queue")
		if (vec_ptr == 0):
			if (jsonformat == True):
				print ("\"error\": \"{}\"".format (errmsg))
			else :
				print (errmsg)
			return

		c = printEventDebugViewVector(" ", vec_ptr.dereference())
		if (jsonformat):
			strout = c.getstring(0, 1)
			if (strout == 'None'):
				strout = ""
			#print ("\"submitted\": \{{}\}".format(strout))
			print (strout)
		else:
			strout = c.getstring(0, 0)
			print (strout)
	
		# free the allocated debug view data
		self.callfunc("appdebug::clFreeAppDebugView",free_args)

obj_ps = printSubmitted ()

class printQueued (infCallUtil):
	"Print the queued commands"

	def invoke (self, arg, jsonformat):
		fargs = []
		fargs.append(gdb.parse_and_eval("(cl_command_queue)"+arg))

		free_args,vec_ptr,errmsg = self.callfunc_verify("appdebug::clPrintCmdQQueued",fargs,"cl_command_queue")
		if (vec_ptr == 0):
			if (jsonformat == True):
				print ("\"error\": \"{}\"".format(errmsg))
			else :
				print (errmsg)
			return

		c = printEventDebugViewVector(" ", vec_ptr.dereference())
		if (jsonformat):
			strout = c.getstring(0, 1)
			if (strout == 'None'):
				strout = ""
			#print ("\"queued\": \{{}\}".format(strout))
			print (strout)
		else:
			strout = c.getstring(0, 0)
			print (strout)

		# free the allocated debug view data
		self.callfunc("appdebug::clFreeAppDebugView",free_args)

obj_pq = printQueued ()

#########################################xmem command#####################################
class printMemInfo (infCallUtil):
	"Print the information about the cl_mem object"
	def invoke (self, arg, jsonformat):
		fargs = []
		try:
			fargs.append(gdb.parse_and_eval("(cl_mem)"+ arg))
		except gdb.error as e:
			print (e.message);
			return

		free_args,clm_ptr,errmsg = self.callfunc_verify("appdebug::clGetMemInfo",fargs,"cl_mem")
		if (clm_ptr == 0):
			if (jsonformat == True):
				print ("\"error\": \"{}\"".format (errmsg))
			else :
				print (errmsg)
			return

		if (jsonformat):
			stdstr = self.callmethod(clm_ptr,"getstring",[0, 1])
			strout = stdstr['_M_dataplus']['_M_p'].string()
			print (strout)
		else :
			stdstr = self.callmethod(clm_ptr,"getstring",[0, 0])
			strout = stdstr['_M_dataplus']['_M_p'].string()
			print (strout)

		# free the allocated vector
		self.callfunc("appdebug::clFreeAppDebugView",free_args)

obj_pm = printMemInfo ()

#########################################xpe command#####################################
class printEventInfo (infCallUtil):
	def invoke (self, arg, from_tty):
		fargs = []
		try:
			fargs.append(gdb.parse_and_eval("(cl_event)"+ arg))
		except gdb.error as e:
			print (e.message);
			return

		free_args,cle_ptr,errmsg = self.callfunc_verify("appdebug::clGetEventInfo",fargs, "cl_event")

		if (cle_ptr == 0):
			print (errmsg)
			return

		stdstr = self.callmethod(cle_ptr,"getstring",[1, 0])
		strout = stdstr['_M_dataplus']['_M_p'].string()
		print (strout)

		# free the allocated vector
		self.callfunc("appdebug::clFreeAppDebugView",free_args)

obj_pe = printEventInfo ()

#########################################xprint command#####################################
class xprintPrefix(gdb.Command):
	"Xilinx command for printing OpenCL runtime data structure"
	def __init__(self):
		super(xprintPrefix, self).__init__(
						"xprint",
						gdb.COMMAND_USER,
						gdb.COMPLETE_COMMAND,
						True)
xprintPrefix()

class xprintQueue (gdb.Command, infCallUtil):
	"Print the command queue contents"
	def __init__ (self):
		super (xprintQueue, self).__init__ ("xprint queue", 
                         gdb.COMMAND_USER)

	def print_one_queue(self, arg, from_tty=True):
		try:
			gdb.parse_and_eval("(cl_command_queue)"+arg)
		except gdb.error as e:
			print (e.message);
			return

		print ("Status:")
		obj_po.invoke(arg, from_tty)
		print ("\nQueued events:")
		obj_pq.invoke(arg, False)
		print ("\nSubmitted events:")
		obj_ps.invoke(arg, False)
		
	def invoke (self, arg="", from_tty=True):
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print (e.message)
			return

		if arg == "":
			#No queues specified, print all queues
			fargs = []
			free_args, vec_ptr, errmsg = self.callfunc_verify("appdebug::clGetCmdQueues",fargs, "cl_command_queue")
			if (vec_ptr == 0):
				print (errmsg)
				return
			vec_val = vec_ptr.dereference()
			start = vec_val['_M_impl']['_M_start']
			finish = vec_val['_M_impl']['_M_finish']
			
			item = start
			print ("Available queues: {}".format(finish-start))
			count = 1
			while (item !=  vec_val['_M_impl']['_M_finish']):
				elem = item.dereference()
				print ("Queue-%d: %s" %(count, elem))
				self.print_one_queue(str(elem), from_tty)
				item=item+1
				if (item !=  vec_val['_M_impl']['_M_finish']):
						print ("") #introduces new line between every queues, except last
				count=count+1
			self.callfunc("appdebug::clFreeAppDebugView",free_args)
		else:
			self.print_one_queue(arg, from_tty)
all_queues=xprintQueue()

class xprintEvent (gdb.Command, infCallUtil):
	"Print the information about the cl_event object"
	def __init__ (self):
		super (xprintEvent, self).__init__ ("xprint event", 
                         gdb.COMMAND_USER)

	def invoke (self, arg, from_tty):
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print (e.message)
			return
		if arg == "" :
			print ("No event argument specified")
			return
		obj_pe.invoke(arg, from_tty)  
xprintEvent()

class xprintMem (gdb.Command,infCallUtil):
	"Print the information about the cl_mem objects"
	def __init__ (self):
		super (xprintMem, self).__init__ ("xprint mem", 
                         gdb.COMMAND_USER)
	def invoke (self, arg="", from_tty=True):
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print (e.message)
			return

		if arg == "":
			#No mem specified, print all mem
			fargs = []

			free_args,vec_ptr, errmsg = self.callfunc_verify("appdebug::clGetClMems",fargs, "cl_mem")
			if (vec_ptr == 0):
				print (errmsg)
				return
			vec_val = vec_ptr.dereference()
			start = vec_val['_M_impl']['_M_start']
			finish = vec_val['_M_impl']['_M_finish']
			print ("Available cl_mems: {}".format(finish-start))
			item = start
			while (item !=  vec_val['_M_impl']['_M_finish']):
				elem = item.dereference()
				obj_pm.invoke(str(elem), False)  
				item = item+1
			self.callfunc("appdebug::clFreeAppDebugView",free_args)
		else: 		
			obj_pm.invoke(arg, from_tty)  
all_mems=xprintMem()

class xprintKernel (gdb.Command,infCallUtil):
	"Print the information about all the submitted kernels"
	def __init__ (self):
		super (xprintKernel, self).__init__ ("xprint kernel", 
                         gdb.COMMAND_USER)
	def invoke (self, arg="", from_tty=True):
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print (e.message)
			return

		if arg == "":
			fargs = []

			free_args, vec_ptr, errmsg = self.callfunc_verify("appdebug::clGetKernelInfo", fargs, "Kernel")
			if (vec_ptr == 0):
				print(errmsg)
				return
			vec_val = vec_ptr.dereference()
			start = vec_val['_M_impl']['_M_start']
			finish = vec_val['_M_impl']['_M_finish']
			print ("Number of Submitted Kernels: {}".format(finish-start))
			item = start
			while (item !=  vec_val['_M_impl']['_M_finish']):
				elem = item.dereference()
				stdstr = self.callmethod(elem,"getstring",[1, 0])
				#strout = unicode(stdstr['_M_dataplus']['_M_p'])
				strout = stdstr['_M_dataplus']['_M_p'].string()
				print (strout)
				item = item+1
			self.callfunc("appdebug::clFreeAppDebugView",free_args)
		else :
			print ("xprint kernel does not accept arguments")
all_kernels=xprintKernel()

class xprintAll (gdb.Command,infCallUtil):
	"Print all command queues cl_mems and kernels"
	def __init__ (self):
		super (xprintAll, self).__init__ ("xprint all", 
                         gdb.COMMAND_USER)
	def invoke (self, arg="", from_tty=True):
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print (e.message)
			return
		all_queues.invoke(arg,from_tty)
		print ("");
		all_mems.invoke(arg,from_tty)
		print ("");
		all_kernels.invoke(arg,from_tty)
xprintAll()

class xprintJSONPrefix(gdb.Command):
	"Xilinx internal command for printing OpenCL runtime data structure in JSON format"
	def __init__(self):
		super(xprintJSONPrefix, self).__init__(
						"xprint_json",
						gdb.COMMAND_USER,
						gdb.COMPLETE_COMMAND,
						True)
xprintJSONPrefix()

class xprintJSONQueue (gdb.Command, infCallUtil):
	"Print the command queue contents"
	def __init__ (self):
		super (xprintJSONQueue, self).__init__ ("xprint_json queue", 
                         gdb.COMMAND_USER)

	def print_one_queue(self, arg, from_tty=False):
		try:
			gdb.parse_and_eval("(cl_command_queue)"+arg)
		except gdb.error as e:
			print ("\"error\": \"{}\"".format(e.message));
			return

		print ("\"name\" : \"Queue-{}\",".format(arg))
		print ("\"queued\": {")
		obj_pq.invoke(arg, True)
		print ("},")

		print ("\"submitted\": {")
		obj_ps.invoke(arg, True)
		print ("}")
		
	def invoke (self, arg="", from_tty=False):
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print ("\"error\": \"{}\"".format (e.message))
			return

		if arg == "":
			#No queues specified, print all queues
			fargs = []
			free_args, vec_ptr, errmsg = self.callfunc_verify("appdebug::clGetCmdQueues",fargs, "cl_command_queue")
			if (vec_ptr == 0):
				print ("\"error\": \"{}\"".format (errmsg))
				return
			vec_val = vec_ptr.dereference()
			start = vec_val['_M_impl']['_M_start']
			finish = vec_val['_M_impl']['_M_finish']
			
			item = start
			count = 1
			while (item !=  vec_val['_M_impl']['_M_finish']):
				if (count > 1) :
					print(",")
				print ("{")
				elem = item.dereference()
				self.print_one_queue(str(elem), False)
				item=item+1
				count=count+1
				print ("}") 
			self.callfunc("appdebug::clFreeAppDebugView",free_args)
		else:
			self.print_one_queue(arg, False)
all_json_queues=xprintJSONQueue()

class xprintJSONMem (gdb.Command,infCallUtil):
	"Print the information about the cl_mem objects"
	def __init__ (self):
		super (xprintJSONMem, self).__init__ ("xprint_json mem", 
                         gdb.COMMAND_USER)

	def print_one_mem(self, arg, from_tty=False):
		print ("\"name\" : \"Mem-{}\",".format(arg))
		print ("\"Details\": {")
		obj_pm.invoke(arg, True)
		print ("}")

	def invoke (self, arg="", from_tty=True):
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print ("\"error\": \"{}\"".format (e.message))
			return

		if arg == "":
			#No mem specified, print all mem
			fargs = []

			free_args,vec_ptr, errmsg = self.callfunc_verify("appdebug::clGetClMems",fargs, "cl_mem")
			if (vec_ptr == 0):
				print ("\"error\": \"{}\"".format (errmsg))
				return
			vec_val = vec_ptr.dereference()
			start = vec_val['_M_impl']['_M_start']
			finish = vec_val['_M_impl']['_M_finish']
			item = start
			count = 1
			while (item !=  vec_val['_M_impl']['_M_finish']):
				if (count > 1) :
					print(",")
				print ("{")
				elem = item.dereference()
				self.print_one_mem(str(elem), False)
				item = item+1
				count=count+1
				print ("}") 
			self.callfunc("appdebug::clFreeAppDebugView",free_args)
		else: 		
			obj_pm.invoke(arg, from_tty)  
all_json_mems=xprintJSONMem()

#import the appdebugint.py which has the xstatus implementation
import appdebugint

class xprintJSONAll (gdb.Command,infCallUtil):
	"Print all command queues, cl_mems  in JSON format"
	def __init__ (self):
		super (xprintJSONAll, self).__init__ ("xprint_json all", 
                         gdb.COMMAND_USER)
	def invoke (self, arg="", from_tty=True):
		print ("{")
		try:
			self.check_app_debug_enabled()
		except ValueError as e:
			print ("\"error\": \"{}\"".format(e.message))
			print ("}")
			return

		print ("\"queues\": [")
		all_json_queues.invoke(arg,from_tty)
		print ("],")

		print ("\"cl_mems\": [")
		all_json_mems.invoke(arg,from_tty)
		print ("],")

		from appdebugint import xstatusJSONAllInfo
		all_json_xstatus=xstatusJSONAllInfo()

		print ("\"debugips\": [")
		all_json_xstatus.invoke(arg,from_tty)
		print ("]")

		print ("}")
xprintJSONAll()
